"""
AvaTax Software Development Kit for Python.

   Copyright 2022 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

    Avalara 1099 & W-9 API Definition
    ## 🔐 Authentication  Use **username/password** or generate a **license key** from: *Avalara Portal → Settings → License and API Keys*.  [More on authentication methods](https://developer.avalara.com/avatax-dm-combined-erp/common-setup/authentication/authentication-methods/)  [Test your credentials](https://developer.avalara.com/avatax/test-credentials/)  ## 📘 API & SDK Documentation  [Avalara SDK (.NET) on GitHub](https://github.com/avadev/Avalara-SDK-DotNet#avalarasdk--the-unified-c-library-for-next-gen-avalara-services)  [Code Examples – 1099 API](https://github.com/avadev/Avalara-SDK-DotNet/blob/main/docs/A1099/V2/Class1099IssuersApi.md#call1099issuersget) 

@author     Sachin Baijal <sachin.baijal@avalara.com>
@author     Jonathan Wenger <jonathan.wenger@avalara.com>
@copyright  2022 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    25.7.2
@link       https://github.com/avadev/AvaTax-REST-V3-Python-SDK
"""

import re  # noqa: F401
import sys  # noqa: F401
import decimal

from Avalara.SDK.api_client import ApiClient, Endpoint as _Endpoint
from Avalara.SDK.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from pydantic import Field, StrictBool, StrictInt, StrictStr
from typing import Optional
from typing_extensions import Annotated
from Avalara.SDK.models.A1099.V2.company_create_update_request_model import CompanyCreateUpdateRequestModel
from Avalara.SDK.models.A1099.V2.company_response import CompanyResponse
from Avalara.SDK.models.A1099.V2.company_response_model import CompanyResponseModel
from Avalara.SDK.models.A1099.V2.paginated_query_result_model_company_response import PaginatedQueryResultModelCompanyResponse
from Avalara.SDK.exceptions import ApiTypeError, ApiValueError, ApiException
from Avalara.SDK.oauth_helper.AvalaraSdkOauthUtils import avalara_retry_oauth

class CompaniesW9Api(object):

    def __init__(self, api_client):
        self.__set_configuration(api_client)
    
    def __verify_api_client(self,api_client):
        if api_client is None:
            raise ApiValueError("APIClient not defined")
    
    def __set_configuration(self, api_client):
        self.__verify_api_client(api_client)
        api_client.set_sdk_version("25.7.2")
        self.api_client = api_client
		
        self.create_company_endpoint = _Endpoint(
            settings={
                'response_type': (CompanyResponseModel,),
                'auth': [
                    'bearer'
                ],
                'endpoint_path': '/w9/companies',
                'operation_id': 'create_company',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'avalara_version',
                    'x_correlation_id',
                    'x_avalara_client',
                    'company_create_update_request_model',
                ],
                'required': [
                    'avalara_version',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'avalara_version':
                        (str,),
                    'x_correlation_id':
                        (str,),
                    'x_avalara_client':
                        (str,),
                    'company_create_update_request_model':
                        (CompanyCreateUpdateRequestModel,),
                },
                'attribute_map': {
                    'avalara_version': 'avalara-version',
                    'x_correlation_id': 'X-Correlation-Id',
                    'x_avalara_client': 'X-Avalara-Client',
                },
                'location_map': {
                    'avalara_version': 'header',
                    'x_correlation_id': 'header',
                    'x_avalara_client': 'header',
                    'company_create_update_request_model': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'avalara-version': '2.0',
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'text/json',
                    'application/*+json'
                ]
            },
            api_client=api_client,
            required_scopes='',
            microservice='A1099'
        )
        self.delete_company_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'bearer'
                ],
                'endpoint_path': '/w9/companies/{id}',
                'operation_id': 'delete_company',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'avalara_version',
                    'x_correlation_id',
                    'x_avalara_client',
                ],
                'required': [
                    'id',
                    'avalara_version',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'avalara_version':
                        (str,),
                    'x_correlation_id':
                        (str,),
                    'x_avalara_client':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                    'avalara_version': 'avalara-version',
                    'x_correlation_id': 'X-Correlation-Id',
                    'x_avalara_client': 'X-Avalara-Client',
                },
                'location_map': {
                    'id': 'path',
                    'avalara_version': 'header',
                    'x_correlation_id': 'header',
                    'x_avalara_client': 'header',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'avalara-version': '2.0',
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            required_scopes='',
            microservice='A1099'
        )
        self.get_companies_endpoint = _Endpoint(
            settings={
                'response_type': (PaginatedQueryResultModelCompanyResponse,),
                'auth': [
                    'bearer'
                ],
                'endpoint_path': '/w9/companies',
                'operation_id': 'get_companies',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'avalara_version',
                    'filter',
                    'top',
                    'skip',
                    'order_by',
                    'count',
                    'count_only',
                    'x_correlation_id',
                    'x_avalara_client',
                ],
                'required': [
                    'avalara_version',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'avalara_version':
                        (str,),
                    'filter':
                        (str,),
                    'top':
                        (int,),
                    'skip':
                        (int,),
                    'order_by':
                        (str,),
                    'count':
                        (bool,),
                    'count_only':
                        (bool,),
                    'x_correlation_id':
                        (str,),
                    'x_avalara_client':
                        (str,),
                },
                'attribute_map': {
                    'avalara_version': 'avalara-version',
                    'filter': '$filter',
                    'top': '$top',
                    'skip': '$skip',
                    'order_by': '$orderBy',
                    'count': 'count',
                    'count_only': 'countOnly',
                    'x_correlation_id': 'X-Correlation-Id',
                    'x_avalara_client': 'X-Avalara-Client',
                },
                'location_map': {
                    'avalara_version': 'header',
                    'filter': 'query',
                    'top': 'query',
                    'skip': 'query',
                    'order_by': 'query',
                    'count': 'query',
                    'count_only': 'query',
                    'x_correlation_id': 'header',
                    'x_avalara_client': 'header',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'avalara-version': '2.0',
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            required_scopes='',
            microservice='A1099'
        )
        self.get_company_endpoint = _Endpoint(
            settings={
                'response_type': (CompanyResponse,),
                'auth': [
                    'bearer'
                ],
                'endpoint_path': '/w9/companies/{id}',
                'operation_id': 'get_company',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'avalara_version',
                    'x_correlation_id',
                    'x_avalara_client',
                ],
                'required': [
                    'id',
                    'avalara_version',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'avalara_version':
                        (str,),
                    'x_correlation_id':
                        (str,),
                    'x_avalara_client':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                    'avalara_version': 'avalara-version',
                    'x_correlation_id': 'X-Correlation-Id',
                    'x_avalara_client': 'X-Avalara-Client',
                },
                'location_map': {
                    'id': 'path',
                    'avalara_version': 'header',
                    'x_correlation_id': 'header',
                    'x_avalara_client': 'header',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'avalara-version': '2.0',
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            required_scopes='',
            microservice='A1099'
        )
        self.update_company_endpoint = _Endpoint(
            settings={
                'response_type': (CompanyResponseModel,),
                'auth': [
                    'bearer'
                ],
                'endpoint_path': '/w9/companies/{id}',
                'operation_id': 'update_company',
                'http_method': 'PUT',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'avalara_version',
                    'x_correlation_id',
                    'x_avalara_client',
                    'company_create_update_request_model',
                ],
                'required': [
                    'id',
                    'avalara_version',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'avalara_version':
                        (str,),
                    'x_correlation_id':
                        (str,),
                    'x_avalara_client':
                        (str,),
                    'company_create_update_request_model':
                        (CompanyCreateUpdateRequestModel,),
                },
                'attribute_map': {
                    'id': 'id',
                    'avalara_version': 'avalara-version',
                    'x_correlation_id': 'X-Correlation-Id',
                    'x_avalara_client': 'X-Avalara-Client',
                },
                'location_map': {
                    'id': 'path',
                    'avalara_version': 'header',
                    'x_correlation_id': 'header',
                    'x_avalara_client': 'header',
                    'company_create_update_request_model': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'avalara-version': '2.0',
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'text/json',
                    'application/*+json'
                ]
            },
            api_client=api_client,
            required_scopes='',
            microservice='A1099'
        )

    @avalara_retry_oauth(max_retry_attempts=2)
    def create_company(
        self,
        avalara_version,
        **kwargs
    ):
        """Creates a new company  # noqa: E501

        Creates a new company  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_company(avalara_version, async_req=True)
        >>> result = thread.get()

        Args:
            avalara_version (str): API version

        Keyword Args:
            x_correlation_id (str): Unique correlation Id in a GUID format. [optional]
            x_avalara_client (str): Identifies the software you are using to call this API. For more information on the client header, see [Client Headers](https://developer.avalara.com/avatax/client-headers/) .. [optional]
            company_create_update_request_model (CompanyCreateUpdateRequestModel): The company to create. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            CompanyResponseModel
                If the method is called asynchronously, returns the request
                thread.
        """
        self.__verify_api_client(self.api_client)
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['avalara_version'] = avalara_version
        return self.create_company_endpoint.call_with_http_info(**kwargs)

    @avalara_retry_oauth(max_retry_attempts=2)
    def delete_company(
        self,
        id,
        avalara_version,
        **kwargs
    ):
        """Deletes a company  # noqa: E501

        Deletes a company  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_company(id, avalara_version, async_req=True)
        >>> result = thread.get()

        Args:
            id (str): The company to delete
            avalara_version (str): API version

        Keyword Args:
            x_correlation_id (str): Unique correlation Id in a GUID format. [optional]
            x_avalara_client (str): Identifies the software you are using to call this API. For more information on the client header, see [Client Headers](https://developer.avalara.com/avatax/client-headers/) .. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        self.__verify_api_client(self.api_client)
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['id'] = id
        kwargs['avalara_version'] = avalara_version
        return self.delete_company_endpoint.call_with_http_info(**kwargs)

    @avalara_retry_oauth(max_retry_attempts=2)
    def get_companies(
        self,
        avalara_version,
        **kwargs
    ):
        """List companies  # noqa: E501

        List existing companies. Filterable/Sortable fields are: \"name\", \"referenceId\", \"group.name\", \"createdAt\" and \"updatedAt\".  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_companies(avalara_version, async_req=True)
        >>> result = thread.get()

        Args:
            avalara_version (str): API version

        Keyword Args:
            filter (str): A filter statement to identify specific records to retrieve.  For more information on filtering, see <a href=\"https://developer.avalara.com/avatax/filtering-in-rest/\">Filtering in REST</a>.. [optional]
            top (int): If zero or greater than 1000, return at most 1000 results.  Otherwise, return this number of results.  Used with skip to provide pagination for large datasets.. [optional]
            skip (int): If nonzero, skip this number of results before returning data. Used with top to provide pagination for large datasets.. [optional]
            order_by (str): A comma separated list of sort statements in the format (fieldname) [ASC|DESC], for example id ASC.. [optional]
            count (bool): If true, return the global count of elements in the collection.. [optional]
            count_only (bool): If true, return ONLY the global count of elements in the collection.  It only applies when count=true.. [optional]
            x_correlation_id (str): Unique correlation Id in a GUID format. [optional]
            x_avalara_client (str): Identifies the software you are using to call this API. For more information on the client header, see [Client Headers](https://developer.avalara.com/avatax/client-headers/) .. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            PaginatedQueryResultModelCompanyResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        self.__verify_api_client(self.api_client)
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['avalara_version'] = avalara_version
        return self.get_companies_endpoint.call_with_http_info(**kwargs)

    @avalara_retry_oauth(max_retry_attempts=2)
    def get_company(
        self,
        id,
        avalara_version,
        **kwargs
    ):
        """Retrieve a company  # noqa: E501

        Retrieve an existing company  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_company(id, avalara_version, async_req=True)
        >>> result = thread.get()

        Args:
            id (str): Id of the company
            avalara_version (str): API version

        Keyword Args:
            x_correlation_id (str): Unique correlation Id in a GUID format. [optional]
            x_avalara_client (str): Identifies the software you are using to call this API. For more information on the client header, see [Client Headers](https://developer.avalara.com/avatax/client-headers/) .. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            CompanyResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        self.__verify_api_client(self.api_client)
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['id'] = id
        kwargs['avalara_version'] = avalara_version
        return self.get_company_endpoint.call_with_http_info(**kwargs)

    @avalara_retry_oauth(max_retry_attempts=2)
    def update_company(
        self,
        id,
        avalara_version,
        **kwargs
    ):
        """Update a company  # noqa: E501

        Update a company  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_company(id, avalara_version, async_req=True)
        >>> result = thread.get()

        Args:
            id (str): The ID of the company to update
            avalara_version (str): API version

        Keyword Args:
            x_correlation_id (str): Unique correlation Id in a GUID format. [optional]
            x_avalara_client (str): Identifies the software you are using to call this API. For more information on the client header, see [Client Headers](https://developer.avalara.com/avatax/client-headers/) .. [optional]
            company_create_update_request_model (CompanyCreateUpdateRequestModel): The updated company data. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            CompanyResponseModel
                If the method is called asynchronously, returns the request
                thread.
        """
        self.__verify_api_client(self.api_client)
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['id'] = id
        kwargs['avalara_version'] = avalara_version
        return self.update_company_endpoint.call_with_http_info(**kwargs)




# coding: utf-8

"""
AvaTax Software Development Kit for Python.

   Copyright 2022 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

    Avalara 1099 & W-9 API Definition
    ## 🔐 Authentication  Use **username/password** or generate a **license key** from: *Avalara Portal → Settings → License and API Keys*.  [More on authentication methods](https://developer.avalara.com/avatax-dm-combined-erp/common-setup/authentication/authentication-methods/)  [Test your credentials](https://developer.avalara.com/avatax/test-credentials/)  ## 📘 API & SDK Documentation  [Avalara SDK (.NET) on GitHub](https://github.com/avadev/Avalara-SDK-DotNet#avalarasdk--the-unified-c-library-for-next-gen-avalara-services)  [Code Examples – 1099 API](https://github.com/avadev/Avalara-SDK-DotNet/blob/main/docs/A1099/V2/Class1099IssuersApi.md#call1099issuersget) 

@author     Sachin Baijal <sachin.baijal@avalara.com>
@author     Jonathan Wenger <jonathan.wenger@avalara.com>
@copyright  2022 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    25.7.0
@link       https://github.com/avadev/AvaTax-REST-V3-Python-SDK
"""

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class FormRequestModel(BaseModel):
    """
    FormRequestModel
    """ # noqa: E501
    id: Optional[StrictStr] = None
    type: Optional[StrictStr] = None
    form_type: Optional[StrictStr] = Field(default=None, description="\"W9\" is currently the only supported value", alias="formType")
    company_id: Optional[StrictInt] = Field(default=None, description="Track1099's ID of your company, found in the W-9 UI", alias="companyId")
    company_name: Optional[StrictStr] = Field(default=None, description="Name of your company, set in the W-9 UI", alias="companyName")
    company_email: Optional[StrictStr] = Field(default=None, description="Contact email of your company, set in the W-9 UI", alias="companyEmail")
    reference_id: Optional[StrictStr] = Field(default=None, description="Your internal identifier for the vendor from whom you are requesting a form", alias="referenceId")
    signed_at: Optional[datetime] = Field(default=None, description="The timestamp this vendor (identified by your ReferenceId) last signed a complete W-9, null if you did not include a ReferenceId or the vendor has not yet signed a W-9 in Track1099", alias="signedAt")
    tin_match_status: Optional[StrictStr] = Field(default=None, description="Result of IRS TIN match query for name and TIN in the last signed form, null if signed_at is null", alias="tinMatchStatus")
    expires_at: Optional[datetime] = Field(default=None, description="Timestamp when this FormRequest will expire, ttl (or 3600) seconds from creation", alias="expiresAt")
    signed_pdf: Optional[StrictStr] = Field(default=None, description="URL of PDF representation of just-signed form, otherwise null. Integrations may use this value to offer a \"download for your records\" function after a vendor completes and signs a form. Link expires at the same time as this FormRequest. Treat the format of this URL as opaque and expect it to change in the future.", alias="signedPdf")
    __properties: ClassVar[List[str]] = []

    @field_validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['FormRequest']):
            raise ValueError("must be one of enum values ('FormRequest')")
        return value

    @field_validator('form_type')
    def form_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['W9']):
            raise ValueError("must be one of enum values ('W9')")
        return value

    @field_validator('tin_match_status')
    def tin_match_status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['None', 'Matched']):
            raise ValueError("must be one of enum values ('None', 'Matched')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of FormRequestModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of FormRequestModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
        })
        return _obj





# coding: utf-8

"""
AvaTax Software Development Kit for Python.

   Copyright 2022 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

    Avalara 1099 & W-9 API Definition
    ## 🔐 Authentication  Use **username/password** or generate a **license key** from: *Avalara Portal → Settings → License and API Keys*.  [More on authentication methods](https://developer.avalara.com/avatax-dm-combined-erp/common-setup/authentication/authentication-methods/)  [Test your credentials](https://developer.avalara.com/avatax/test-credentials/)  ## 📘 API & SDK Documentation  [Avalara SDK (.NET) on GitHub](https://github.com/avadev/Avalara-SDK-DotNet#avalarasdk--the-unified-c-library-for-next-gen-avalara-services)  [Code Examples – 1099 API](https://github.com/avadev/Avalara-SDK-DotNet/blob/main/docs/A1099/V2/Class1099IssuersApi.md#call1099issuersget) 

@author     Sachin Baijal <sachin.baijal@avalara.com>
@author     Jonathan Wenger <jonathan.wenger@avalara.com>
@copyright  2022 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    25.7.2
@link       https://github.com/avadev/AvaTax-REST-V3-Python-SDK
"""

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set
from typing_extensions import Self

class W4FormResponse(BaseModel):
    """
    W4FormResponse
    """ # noqa: E501
    employee_first_name: Optional[StrictStr] = Field(default=None, description="The first name of the employee.", alias="employeeFirstName")
    employee_middle_name: Optional[StrictStr] = Field(default=None, description="The middle name of the employee.", alias="employeeMiddleName")
    employee_last_name: Optional[StrictStr] = Field(default=None, description="The last name of the employee.", alias="employeeLastName")
    employee_name_suffix: Optional[StrictStr] = Field(default=None, description="The name suffix of the employee.", alias="employeeNameSuffix")
    tin_type: Optional[StrictStr] = Field(default=None, description="The type of TIN provided.", alias="tinType")
    tin: Optional[StrictStr] = Field(default=None, description="The taxpayer identification number (TIN).")
    address: Optional[StrictStr] = Field(default=None, description="The address of the employee.")
    city: Optional[StrictStr] = Field(default=None, description="The city of residence of the employee.")
    state: Optional[StrictStr] = Field(default=None, description="The state of residence of the employee.")
    zip: Optional[StrictStr] = Field(default=None, description="The ZIP code of residence of the employee.")
    marital_status: Optional[StrictStr] = Field(default=None, description="The marital status of the employee.", alias="maritalStatus")
    last_name_differs: Optional[StrictBool] = Field(default=None, description="Indicates whether the last name differs from prior records.", alias="lastNameDiffers")
    num_allowances: Optional[StrictInt] = Field(default=None, description="The number of allowances claimed by the employee.", alias="numAllowances")
    other_dependents: Optional[StrictInt] = Field(default=None, description="The number of dependents other than allowances.", alias="otherDependents")
    non_job_income: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of non-job income.", alias="nonJobIncome")
    deductions: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of deductions claimed.")
    additional_withheld: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The additional amount withheld.", alias="additionalWithheld")
    exempt_from_withholding: Optional[StrictBool] = Field(default=None, description="Indicates whether the employee is exempt from withholding.", alias="exemptFromWithholding")
    office_code: Optional[StrictStr] = Field(default=None, description="The office code associated with the form.", alias="officeCode")
    id: Optional[StrictStr] = Field(default=None, description="The unique identifier for the form.")
    type: Optional[StrictStr] = Field(default=None, description="The form type.")
    entry_status: Optional[StrictStr] = Field(default=None, description="The form status.", alias="entryStatus")
    entry_status_date: Optional[datetime] = Field(default=None, description="The timestamp for the latest status update.", alias="entryStatusDate")
    reference_id: Optional[StrictStr] = Field(default=None, description="A reference identifier for the form.", alias="referenceId")
    company_id: Optional[StrictStr] = Field(default=None, description="The ID of the associated company.", alias="companyId")
    display_name: Optional[StrictStr] = Field(default=None, description="The display name associated with the form.", alias="displayName")
    email: Optional[StrictStr] = Field(default=None, description="The email address of the individual associated with the form.")
    archived: Optional[StrictBool] = Field(default=None, description="Indicates whether the form is archived.")
    signature: Optional[StrictStr] = Field(default=None, description="The signature of the form.")
    signed_date: Optional[datetime] = Field(default=None, description="The date the form was signed.", alias="signedDate")
    e_delivery_consented_at: Optional[datetime] = Field(default=None, description="The date when e-delivery was consented.", alias="eDeliveryConsentedAt")
    created_at: Optional[datetime] = Field(default=None, description="The creation date of the form.", alias="createdAt")
    updated_at: Optional[datetime] = Field(default=None, description="The last updated date of the form.", alias="updatedAt")
    __properties: ClassVar[List[str]] = ["id", "type", "entryStatus", "entryStatusDate", "referenceId", "companyId", "displayName", "email", "archived", "signature", "signedDate", "eDeliveryConsentedAt", "createdAt", "updatedAt"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of W4FormResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if entry_status_date (nullable) is None
        # and model_fields_set contains the field
        if self.entry_status_date is None and "entry_status_date" in self.model_fields_set:
            _dict['entryStatusDate'] = None

        # set to None if reference_id (nullable) is None
        # and model_fields_set contains the field
        if self.reference_id is None and "reference_id" in self.model_fields_set:
            _dict['referenceId'] = None

        # set to None if email (nullable) is None
        # and model_fields_set contains the field
        if self.email is None and "email" in self.model_fields_set:
            _dict['email'] = None

        # set to None if signature (nullable) is None
        # and model_fields_set contains the field
        if self.signature is None and "signature" in self.model_fields_set:
            _dict['signature'] = None

        # set to None if signed_date (nullable) is None
        # and model_fields_set contains the field
        if self.signed_date is None and "signed_date" in self.model_fields_set:
            _dict['signedDate'] = None

        # set to None if e_delivery_consented_at (nullable) is None
        # and model_fields_set contains the field
        if self.e_delivery_consented_at is None and "e_delivery_consented_at" in self.model_fields_set:
            _dict['eDeliveryConsentedAt'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of W4FormResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "type": obj.get("type"),
            "entryStatus": obj.get("entryStatus"),
            "entryStatusDate": obj.get("entryStatusDate"),
            "referenceId": obj.get("referenceId"),
            "companyId": obj.get("companyId"),
            "displayName": obj.get("displayName"),
            "email": obj.get("email"),
            "archived": obj.get("archived"),
            "signature": obj.get("signature"),
            "signedDate": obj.get("signedDate"),
            "eDeliveryConsentedAt": obj.get("eDeliveryConsentedAt"),
            "createdAt": obj.get("createdAt"),
            "updatedAt": obj.get("updatedAt")
        })
        return _obj





# coding: utf-8

"""
AvaTax Software Development Kit for Python.

   Copyright 2022 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

    Avalara E-Invoicing API
    An API that supports sending data for an E-Invoicing compliance use-case. 

@author     Sachin Baijal <sachin.baijal@avalara.com>
@author     Jonathan Wenger <jonathan.wenger@avalara.com>
@copyright  2022 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    25.7.2
@link       https://github.com/avadev/AvaTax-REST-V3-Python-SDK
"""

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from Avalara.SDK.models.EInvoicing.V1.input_data_formats import InputDataFormats
from Avalara.SDK.models.EInvoicing.V1.output_data_formats import OutputDataFormats
from Avalara.SDK.models.EInvoicing.V1.workflow_ids import WorkflowIds
from typing import Optional, Set
from typing_extensions import Self

class Mandate(BaseModel):
    """
    An object representing the country mandate
    """ # noqa: E501
    mandate_id: Optional[StrictStr] = Field(default=None, description="The `mandateId` is comprised of the country code, mandate type, and the network or regulation type (for example, AU-B2G-PEPPOL). Keep in mind the following when specifying a `mandateId`. - A country can have multiple mandate types (B2C, B2B, B2G). - A entity/company can opt in for multiple mandates. - A `mandateId` is the combination of country + mandate type + network/regulation.", alias="mandateId")
    country_mandate: Optional[StrictStr] = Field(default=None, description="**[LEGACY]** This field is retained for backward compatibility. It is recommended to use `mandateId` instead. The `countryMandate` similar to the `mandateId` is comprised of the country code, mandate type, and the network or regulation type (for example, AU-B2G-PEPPOL). ", alias="countryMandate")
    country_code: Optional[StrictStr] = Field(default=None, description="Country code", alias="countryCode")
    description: Optional[StrictStr] = Field(default=None, description="Mandate description")
    supported_by_elrapi: Optional[StrictBool] = Field(default=None, description="Indicates whether this mandate supported by the ELR API", alias="supportedByELRAPI")
    mandate_format: Optional[StrictStr] = Field(default=None, description="Mandate format", alias="mandateFormat")
    e_invoicing_flow: Optional[StrictStr] = Field(default=None, description="The type of e-invoicing flow for this mandate", alias="eInvoicingFlow")
    e_invoicing_flow_documentation_link: Optional[StrictStr] = Field(default=None, description="Link to the documentation for this mandate's e-invoicing flow", alias="eInvoicingFlowDocumentationLink")
    get_invoice_available_media_type: Optional[List[StrictStr]] = Field(default=None, description="List of available media types for downloading invoices for this mandate", alias="getInvoiceAvailableMediaType")
    supports_inbound_digital_document: Optional[StrictStr] = Field(default=None, description="Indicates whether this mandate supports inbound digital documents", alias="supportsInboundDigitalDocument")
    input_data_formats: Optional[List[InputDataFormats]] = Field(default=None, description="Format and version used when inputting the data", alias="inputDataFormats")
    output_data_formats: Optional[List[OutputDataFormats]] = Field(default=None, description="Lists the supported output document formats for the country mandate. For countries where specifying an output document format is required (e.g., France), this array will contain the applicable formats. For other countries where output format selection is not necessary, the array will be empty.", alias="outputDataFormats")
    workflow_ids: Optional[List[WorkflowIds]] = Field(default=None, description="Workflow ID list", alias="workflowIds")
    __properties: ClassVar[List[str]] = ["mandateId", "countryMandate", "countryCode", "description", "supportedByELRAPI", "mandateFormat", "eInvoicingFlow", "eInvoicingFlowDocumentationLink", "getInvoiceAvailableMediaType", "supportsInboundDigitalDocument", "inputDataFormats", "outputDataFormats", "workflowIds"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Mandate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in input_data_formats (list)
        _items = []
        if self.input_data_formats:
            for _item in self.input_data_formats:
                if _item:
                    _items.append(_item.to_dict())
            _dict['inputDataFormats'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in output_data_formats (list)
        _items = []
        if self.output_data_formats:
            for _item in self.output_data_formats:
                if _item:
                    _items.append(_item.to_dict())
            _dict['outputDataFormats'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in workflow_ids (list)
        _items = []
        if self.workflow_ids:
            for _item in self.workflow_ids:
                if _item:
                    _items.append(_item.to_dict())
            _dict['workflowIds'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Mandate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "mandateId": obj.get("mandateId"),
            "countryMandate": obj.get("countryMandate"),
            "countryCode": obj.get("countryCode"),
            "description": obj.get("description"),
            "supportedByELRAPI": obj.get("supportedByELRAPI"),
            "mandateFormat": obj.get("mandateFormat"),
            "eInvoicingFlow": obj.get("eInvoicingFlow"),
            "eInvoicingFlowDocumentationLink": obj.get("eInvoicingFlowDocumentationLink"),
            "getInvoiceAvailableMediaType": obj.get("getInvoiceAvailableMediaType"),
            "supportsInboundDigitalDocument": obj.get("supportsInboundDigitalDocument"),
            "inputDataFormats": [InputDataFormats.from_dict(_item) for _item in obj["inputDataFormats"]] if obj.get("inputDataFormats") is not None else None,
            "outputDataFormats": [OutputDataFormats.from_dict(_item) for _item in obj["outputDataFormats"]] if obj.get("outputDataFormats") is not None else None,
            "workflowIds": [WorkflowIds.from_dict(_item) for _item in obj["workflowIds"]] if obj.get("workflowIds") is not None else None
        })
        return _obj





# coding: utf-8

"""
AvaTax Software Development Kit for Python.

   Copyright 2022 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

    Avalara 1099 & W-9 API Definition
    ## 🔐 Authentication  Use **username/password** or generate a **license key** from: *Avalara Portal → Settings → License and API Keys*.  [More on authentication methods](https://developer.avalara.com/avatax-dm-combined-erp/common-setup/authentication/authentication-methods/)  [Test your credentials](https://developer.avalara.com/avatax/test-credentials/)  ## 📘 API & SDK Documentation  [Avalara SDK (.NET) on GitHub](https://github.com/avadev/Avalara-SDK-DotNet#avalarasdk--the-unified-c-library-for-next-gen-avalara-services)  [Code Examples – 1099 API](https://github.com/avadev/Avalara-SDK-DotNet/blob/main/docs/A1099/V2/Class1099IssuersApi.md#call1099issuersget) 

@author     Sachin Baijal <sachin.baijal@avalara.com>
@author     Jonathan Wenger <jonathan.wenger@avalara.com>
@copyright  2022 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    25.7.2
@link       https://github.com/avadev/AvaTax-REST-V3-Python-SDK
"""

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from Avalara.SDK.models.A1099.V2.w8_ben_e_substantial_us_owner_data_model import W8BenESubstantialUsOwnerDataModel
from typing import Optional, Set
from typing_extensions import Self

class W8BenFormDataModel(BaseModel):
    """
    W-8 BEN form
    """ # noqa: E501
    type: Optional[StrictStr] = Field(default=None, description="Type of the form, always W8ben for this model.")
    signed_date: Optional[datetime] = Field(default=None, description="The date the form was signed.", alias="signedDate")
    birthday: Optional[StrictStr] = Field(default=None, description="The birthday of the individual associated with the form.")
    foreign_tin_not_required: Optional[StrictBool] = Field(default=None, description="Indicates whether a foreign TIN is not required.", alias="foreignTinNotRequired")
    archived: Optional[StrictBool] = Field(default=None, description="Indicates whether the form is archived.")
    reference_id: Optional[StrictStr] = Field(default=None, description="A reference identifier for the form.", alias="referenceId")
    company_id: Optional[StrictInt] = Field(default=None, description="The ID of the associated company.", alias="companyId")
    display_name: Optional[StrictStr] = Field(default=None, description="The display name associated with the form.", alias="displayName")
    email: Optional[StrictStr] = Field(default=None, description="The email address of the individual associated with the form.")
    type_of_tin: Optional[StrictStr] = Field(default=None, description="The type of TIN provided.", alias="typeOfTin")
    name: Optional[StrictStr] = Field(default=None, description="The name of the individual or entity associated with the form.")
    residence_address: Optional[StrictStr] = Field(default=None, description="The residential address of the individual or entity.", alias="residenceAddress")
    residence_city: Optional[StrictStr] = Field(default=None, description="The city of residence.", alias="residenceCity")
    residence_state: Optional[StrictStr] = Field(default=None, description="The state of residence.", alias="residenceState")
    residence_postal_code: Optional[StrictStr] = Field(default=None, description="The postal code of the residence.", alias="residencePostalCode")
    residence_country: Optional[StrictStr] = Field(default=None, description="The country of residence.", alias="residenceCountry")
    residence_is_mailing: Optional[StrictBool] = Field(default=None, alias="residenceIsMailing")
    mailing_address: Optional[StrictStr] = Field(default=None, description="The mailing address.", alias="mailingAddress")
    mailing_city: Optional[StrictStr] = Field(default=None, description="The city of the mailing address.", alias="mailingCity")
    mailing_state: Optional[StrictStr] = Field(default=None, description="The state of the mailing address.", alias="mailingState")
    mailing_postal_code: Optional[StrictStr] = Field(default=None, description="The postal code of the mailing address.", alias="mailingPostalCode")
    mailing_country: Optional[StrictStr] = Field(default=None, description="The country of the mailing address.", alias="mailingCountry")
    tin: Optional[StrictStr] = Field(default=None, description="The taxpayer identification number (TIN).")
    foreign_tin: Optional[StrictStr] = Field(default=None, description="The foreign taxpayer identification number (TIN).", alias="foreignTin")
    reference_number: Optional[StrictStr] = Field(default=None, description="A reference number for the form.", alias="referenceNumber")
    citizenship_country: Optional[StrictStr] = Field(default=None, description="The country of citizenship.", alias="citizenshipCountry")
    treaty_country: Optional[StrictStr] = Field(default=None, description="The country for which the treaty applies.", alias="treatyCountry")
    treaty_article: Optional[StrictStr] = Field(default=None, description="The specific article of the treaty being claimed.", alias="treatyArticle")
    withholding_rate: Optional[StrictStr] = Field(default=None, description="The withholding rate applied as per the treaty.", alias="withholdingRate")
    income_type: Optional[StrictStr] = Field(default=None, description="The type of income covered by the treaty.", alias="incomeType")
    treaty_reasons: Optional[StrictStr] = Field(default=None, description="The reasons for claiming treaty benefits.", alias="treatyReasons")
    signer_name: Optional[StrictStr] = Field(default=None, description="The name of the signer of the form.", alias="signerName")
    signer_capacity: Optional[StrictStr] = Field(default=None, description="The capacity in which the signer is signing the form.", alias="signerCapacity")
    e_delivery_consented_at: Optional[datetime] = Field(default=None, description="The date when e-delivery was consented.", alias="eDeliveryConsentedAt")
    created_at: Optional[datetime] = Field(default=None, description="The creation date of the form.", alias="createdAt")
    updated_at: Optional[datetime] = Field(default=None, description="The last updated date of the form.", alias="updatedAt")
    employee_first_name: Optional[StrictStr] = Field(default=None, description="The first name of the employee.", alias="employeeFirstName")
    employee_middle_name: Optional[StrictStr] = Field(default=None, description="The middle name of the employee.", alias="employeeMiddleName")
    employee_last_name: Optional[StrictStr] = Field(default=None, description="The last name of the employee.", alias="employeeLastName")
    employee_name_suffix: Optional[StrictStr] = Field(default=None, description="The name suffix of the employee.", alias="employeeNameSuffix")
    address: Optional[StrictStr] = Field(default=None, description="The address of the individual or entity.")
    city: Optional[StrictStr] = Field(default=None, description="The city of the address.")
    state: Optional[StrictStr] = Field(default=None, description="The state of the address.")
    zip: Optional[StrictStr] = Field(default=None, description="The ZIP code of the address.")
    box3_marital_status: Optional[StrictStr] = Field(default=None, description="The marital status of the employee.", alias="box3MaritalStatus")
    box4_last_name_differs: Optional[StrictBool] = Field(default=None, description="Indicates whether the last name differs from prior records.", alias="box4LastNameDiffers")
    box5_num_allowances: Optional[StrictInt] = Field(default=None, description="The number of allowances claimed by the employee.", alias="box5NumAllowances")
    other_dependents: Optional[StrictInt] = Field(default=None, description="The number of dependents other than allowances.", alias="otherDependents")
    non_job_income: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of non-job income.", alias="nonJobIncome")
    deductions: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of deductions claimed.")
    box6_additional_withheld: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The additional amount withheld.", alias="box6AdditionalWithheld")
    box7_exempt_from_withholding: Optional[StrictBool] = Field(default=None, description="Indicates whether the employee is exempt from withholding.", alias="box7ExemptFromWithholding")
    office_code: Optional[StrictStr] = Field(default=None, description="The office code associated with the form.", alias="officeCode")
    disregarded_entity_chapter4_fatca_status: Optional[StrictStr] = Field(default=None, description="The FATCA status for disregarded entities under Chapter 4.", alias="disregardedEntityChapter4FatcaStatus")
    disregarded_address: Optional[StrictStr] = Field(default=None, description="The address for disregarded entities.", alias="disregardedAddress")
    disregarded_city: Optional[StrictStr] = Field(default=None, description="The city for disregarded entities.", alias="disregardedCity")
    disregarded_state: Optional[StrictStr] = Field(default=None, description="The state for disregarded entities.", alias="disregardedState")
    disregarded_postal_code: Optional[StrictStr] = Field(default=None, description="The postal code for disregarded entities.", alias="disregardedPostalCode")
    disregarded_country: Optional[StrictStr] = Field(default=None, description="The country for disregarded entities.", alias="disregardedCountry")
    ftin_not_required: Optional[StrictBool] = Field(default=None, description="Indicates whether a foreign TIN is not required.", alias="ftinNotRequired")
    giin: Optional[StrictStr] = Field(default=None, description="The global intermediary identification number (GIIN).")
    chapter3_entity_type: Optional[StrictStr] = Field(default=None, description="The Chapter 3 entity type.", alias="chapter3EntityType")
    chapter4_fatca_status: Optional[StrictStr] = Field(default=None, description="The Chapter 4 FATCA status.", alias="chapter4FatcaStatus")
    disregarded_entity: Optional[StrictStr] = Field(default=None, description="The disregarded entity information.", alias="disregardedEntity")
    disregarded_entity_giin: Optional[StrictStr] = Field(default=None, description="The GIIN for disregarded entities.", alias="disregardedEntityGiin")
    benefit_limitation: Optional[StrictStr] = Field(default=None, description="The benefit limitation for tax treaty claims.", alias="benefitLimitation")
    part4_sponsoring_entity: Optional[StrictStr] = Field(default=None, description="The sponsoring entity information for Part 4.", alias="part4SponsoringEntity")
    part4_sponsoring_entity_giin: Optional[StrictStr] = Field(default=None, description="The GIIN for the sponsoring entity in Part 4.", alias="part4SponsoringEntityGiin")
    part7_sponsoring_entity: Optional[StrictStr] = Field(default=None, description="The sponsoring entity information for Part 7.", alias="part7SponsoringEntity")
    part12_iga_country: Optional[StrictStr] = Field(default=None, description="The IGA country information for Part 12.", alias="part12IgaCountry")
    part12_iga_type: Optional[StrictStr] = Field(default=None, description="The IGA type information for Part 12.", alias="part12IgaType")
    part12_fatca_status_under_iga_annex_ii: Optional[StrictStr] = Field(default=None, description="The FATCA status under IGA Annex II for Part 12.", alias="part12FatcaStatusUnderIgaAnnexIi")
    part12_trustee_name: Optional[StrictStr] = Field(default=None, description="The trustee name for Part 12.", alias="part12TrusteeName")
    part12_trustee_is_foreign: Optional[StrictBool] = Field(default=None, description="Indicates whether the trustee is foreign for Part 12.", alias="part12TrusteeIsForeign")
    part12_model2_iga_giin: Optional[StrictStr] = Field(default=None, description="The GIIN for Model 2 IGA in Part 12.", alias="part12Model2IgaGiin")
    box37_a_exchange: Optional[StrictStr] = Field(default=None, description="The exchange information for Box 37A.", alias="box37AExchange")
    box37_b_exchange: Optional[StrictStr] = Field(default=None, description="The exchange information for Box 37B.", alias="box37BExchange")
    box37_b_entity: Optional[StrictStr] = Field(default=None, description="The entity information for Box 37B.", alias="box37BEntity")
    part28_sponsoring_entity: Optional[StrictStr] = Field(default=None, description="The sponsoring entity information for Part 28.", alias="part28SponsoringEntity")
    part28_sponsoring_entity_giin: Optional[StrictStr] = Field(default=None, description="The GIIN for the sponsoring entity in Part 28.", alias="part28SponsoringEntityGiin")
    making_treaty_claim: Optional[StrictBool] = Field(default=None, alias="makingTreatyClaim")
    certify_box14_a: Optional[StrictBool] = Field(default=None, alias="certifyBox14A")
    certify_box14_b: Optional[StrictBool] = Field(default=None, alias="certifyBox14B")
    certify_box14_c: Optional[StrictBool] = Field(default=None, alias="certifyBox14C")
    certify_box17_1: Optional[StrictBool] = Field(default=None, alias="certifyBox17_1")
    certify_box17_2: Optional[StrictBool] = Field(default=None, alias="certifyBox17_2")
    certify_box18: Optional[StrictBool] = Field(default=None, alias="certifyBox18")
    certify_box19: Optional[StrictBool] = Field(default=None, alias="certifyBox19")
    certify_box21: Optional[StrictBool] = Field(default=None, alias="certifyBox21")
    certify_box22: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 22.", alias="certifyBox22")
    certify_box23: Optional[StrictBool] = Field(default=None, alias="certifyBox23")
    certify_box24_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 24A.", alias="certifyBox24A")
    certify_box24_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 24B.", alias="certifyBox24B")
    certify_box24_c: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 24C.", alias="certifyBox24C")
    certify_box24_d: Optional[StrictBool] = Field(default=None, alias="certifyBox24D")
    certify_box25_a: Optional[StrictBool] = Field(default=None, alias="certifyBox25A")
    certify_box25_b: Optional[StrictBool] = Field(default=None, alias="certifyBox25B")
    certify_box25_c: Optional[StrictBool] = Field(default=None, alias="certifyBox25C")
    certify_box26: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 26.", alias="certifyBox26")
    certify_box27: Optional[StrictBool] = Field(default=None, alias="certifyBox27")
    certify_box28_a: Optional[StrictBool] = Field(default=None, alias="certifyBox28A")
    certify_box28_b: Optional[StrictBool] = Field(default=None, alias="certifyBox28B")
    certify_box29_a: Optional[StrictBool] = Field(default=None, alias="certifyBox29A")
    certify_box29_b: Optional[StrictBool] = Field(default=None, alias="certifyBox29B")
    certify_box29_c: Optional[StrictBool] = Field(default=None, alias="certifyBox29C")
    certify_box29_d: Optional[StrictBool] = Field(default=None, alias="certifyBox29D")
    certify_box29_e: Optional[StrictBool] = Field(default=None, alias="certifyBox29E")
    certify_box29_f: Optional[StrictBool] = Field(default=None, alias="certifyBox29F")
    certify_box30: Optional[StrictBool] = Field(default=None, alias="certifyBox30")
    certify_box31: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 31.", alias="certifyBox31")
    certify_box32: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 32.", alias="certifyBox32")
    certify_box33: Optional[StrictBool] = Field(default=None, alias="certifyBox33")
    certify_box34: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 34.", alias="certifyBox34")
    certify_box35: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 35.", alias="certifyBox35")
    certify_box36: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 36.", alias="certifyBox36")
    certify_box37_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 37A.", alias="certifyBox37A")
    certify_box37_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 37B.", alias="certifyBox37B")
    certify_box38: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 38.", alias="certifyBox38")
    certify_box39: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 39.", alias="certifyBox39")
    certify_box40_a: Optional[StrictBool] = Field(default=None, alias="certifyBox40A")
    certify_box40_b: Optional[StrictBool] = Field(default=None, alias="certifyBox40B")
    certify_box40_c: Optional[StrictBool] = Field(default=None, alias="certifyBox40C")
    certify_box41: Optional[StrictBool] = Field(default=None, alias="certifyBox41")
    certify_box43: Optional[StrictBool] = Field(default=None, alias="certifyBox43")
    certify_part29_signature: Optional[StrictBool] = Field(default=None, alias="certifyPart29Signature")
    part19_formation_or_resolution_date: Optional[datetime] = Field(default=None, alias="part19FormationOrResolutionDate")
    part20_filing_date: Optional[datetime] = Field(default=None, alias="part20FilingDate")
    part21_determination_date: Optional[datetime] = Field(default=None, alias="part21DeterminationDate")
    substantial_us_owners: Optional[List[W8BenESubstantialUsOwnerDataModel]] = Field(default=None, alias="substantialUsOwners")
    ein: Optional[StrictStr] = Field(default=None, description="The employer identification number (EIN).")
    ein_type: Optional[StrictStr] = Field(default=None, description="The type of employer identification number (EIN).", alias="einType")
    certify_box14: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 14.", alias="certifyBox14")
    certify_box15_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 15A.", alias="certifyBox15A")
    certify_box15_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 15B.", alias="certifyBox15B")
    certify_box15_c: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 15C.", alias="certifyBox15C")
    certify_box15_d: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 15D.", alias="certifyBox15D")
    certify_box15_e: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 15E.", alias="certifyBox15E")
    certify_box15_f: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 15F.", alias="certifyBox15F")
    certify_box15_g: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 15G.", alias="certifyBox15G")
    certify_box15_h: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 15H.", alias="certifyBox15H")
    certify_box15_i: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 15I.", alias="certifyBox15I")
    certify_box16_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 16A.", alias="certifyBox16A")
    box16_b_qdd_corporate: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 16B as a QDD corporate entity.", alias="box16BQddCorporate")
    box16_b_qdd_partnership: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 16B as a QDD partnership.", alias="box16BQddPartnership")
    box16_b_qdd_disregarded_entity: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 16B as a QDD disregarded entity.", alias="box16BQddDisregardedEntity")
    certify_box17_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 17A.", alias="certifyBox17A")
    certify_box17_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 17B.", alias="certifyBox17B")
    certify_box17_c: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 17C.", alias="certifyBox17C")
    certify_box17_d: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 17D.", alias="certifyBox17D")
    certify_box17_e: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 17E.", alias="certifyBox17E")
    certify_box18_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 18A.", alias="certifyBox18A")
    certify_box18_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 18B.", alias="certifyBox18B")
    certify_box18_c: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 18C.", alias="certifyBox18C")
    certify_box18_d: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 18D.", alias="certifyBox18D")
    certify_box18_e: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 18E.", alias="certifyBox18E")
    certify_box18_f: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 18F.", alias="certifyBox18F")
    certify_box19_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 19A.", alias="certifyBox19A")
    certify_box19_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 19B.", alias="certifyBox19B")
    certify_box19_c: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 19C.", alias="certifyBox19C")
    certify_box19_d: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 19D.", alias="certifyBox19D")
    certify_box19_e: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 19E.", alias="certifyBox19E")
    certify_box19_f: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 19F.", alias="certifyBox19F")
    certify_box20: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 20.", alias="certifyBox20")
    certify_box21_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 21A.", alias="certifyBox21A")
    certify_box21_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 21B.", alias="certifyBox21B")
    certify_box21_c: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 21C.", alias="certifyBox21C")
    certify_box21_d: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 21D.", alias="certifyBox21D")
    certify_box21_e: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 21E.", alias="certifyBox21E")
    certify_box21_f: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 21F.", alias="certifyBox21F")
    box23_a_name_sponsoring_entity: Optional[StrictStr] = Field(default=None, description="The name of the sponsoring entity for box 23A.", alias="box23ANameSponsoringEntity")
    certify_box23_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 23B.", alias="certifyBox23B")
    certify_box23_c: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 23C.", alias="certifyBox23C")
    certify_box25: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 25.", alias="certifyBox25")
    box27_a_name_sponsoring_entity: Optional[StrictStr] = Field(default=None, description="The name of the sponsoring entity for box 27A.", alias="box27ANameSponsoringEntity")
    certify_box27_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 27B.", alias="certifyBox27B")
    certify_box28: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 28.", alias="certifyBox28")
    certify_box29: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 29.", alias="certifyBox29")
    certify_box30_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 30A.", alias="certifyBox30A")
    certify_box30_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 30B.", alias="certifyBox30B")
    certify_box30_c: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 30C.", alias="certifyBox30C")
    box32_iga_country: Optional[StrictStr] = Field(default=None, description="The IGA country information for box 32.", alias="box32IgaCountry")
    box32_iga_type: Optional[StrictStr] = Field(default=None, description="The IGA type information for box 32.", alias="box32IgaType")
    box32_iga_treated_as: Optional[StrictStr] = Field(default=None, description="The IGA treatment information for box 32.", alias="box32IgaTreatedAs")
    box32_trustee_or_sponsor: Optional[StrictStr] = Field(default=None, description="The trustee or sponsor information for box 32.", alias="box32TrusteeOrSponsor")
    box32_trustee_is_foreign: Optional[StrictBool] = Field(default=None, description="Indicates whether the trustee is foreign for box 32.", alias="box32TrusteeIsForeign")
    certify_box33_a: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 33A.", alias="certifyBox33A")
    certify_box33_b: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 33B.", alias="certifyBox33B")
    certify_box33_c: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 33C.", alias="certifyBox33C")
    certify_box33_d: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 33D.", alias="certifyBox33D")
    certify_box33_e: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 33E.", alias="certifyBox33E")
    certify_box33_f: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 33F.", alias="certifyBox33F")
    box37_a_securities_market: Optional[StrictStr] = Field(default=None, description="The securities market information for box 37A.", alias="box37ASecuritiesMarket")
    box37_b_name_of_entity: Optional[StrictStr] = Field(default=None, description="The name of the entity for box 37B.", alias="box37BNameOfEntity")
    box37_b_securities_market: Optional[StrictStr] = Field(default=None, description="The securities market information for box 37B.", alias="box37BSecuritiesMarket")
    certify_box40: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 40.", alias="certifyBox40")
    box41_sponsoring_entity: Optional[StrictStr] = Field(default=None, description="The sponsoring entity information for box 41.", alias="box41SponsoringEntity")
    certify_box42: Optional[StrictBool] = Field(default=None, description="Indicates certification for box 42.", alias="certifyBox42")
    box35_formed_on_date: Optional[datetime] = Field(default=None, alias="box35FormedOnDate")
    box36_filed_on_date: Optional[datetime] = Field(default=None, alias="box36FiledOnDate")
    tin_match_status: Optional[StrictStr] = Field(default=None, description="The status of the TIN match.", alias="tinMatchStatus")
    signature: Optional[StrictStr] = Field(default=None, description="The signature itself")
    business_classification: Optional[StrictStr] = Field(default=None, description="The classification of the business.", alias="businessClassification")
    business_name: Optional[StrictStr] = Field(default=None, description="The name of the business associated with the form.", alias="businessName")
    business_other: Optional[StrictStr] = Field(default=None, alias="businessOther")
    exempt_payee_code: Optional[StrictStr] = Field(default=None, alias="exemptPayeeCode")
    exempt_fatca_code: Optional[StrictStr] = Field(default=None, alias="exemptFatcaCode")
    account_number: Optional[StrictStr] = Field(default=None, description="The account number associated with the form.", alias="accountNumber")
    foreign_country_indicator: Optional[StrictBool] = Field(default=None, description="Indicates whether the individual or entity is in a foreign country.", alias="foreignCountryIndicator")
    foreign_address: Optional[StrictStr] = Field(default=None, description="The foreign address of the individual or entity.", alias="foreignAddress")
    backup_withholding: Optional[StrictBool] = Field(default=None, description="Indicates whether backup withholding applies.", alias="backupWithholding")
    is1099able: Optional[StrictBool] = None
    foreign_partner_owner_or_beneficiary: Optional[StrictBool] = Field(default=None, description="Indicates whether the individual is a foreign partner, owner, or beneficiary.", alias="foreignPartnerOwnerOrBeneficiary")
    __properties: ClassVar[List[str]] = ["type", "signedDate", "createdAt", "employeeFirstName", "employeeMiddleName", "employeeLastName", "employeeNameSuffix", "address", "city", "state", "zip", "typeOfTin", "tin", "box3MaritalStatus", "box4LastNameDiffers", "box5NumAllowances", "otherDependents", "nonJobIncome", "deductions", "box6AdditionalWithheld", "box7ExemptFromWithholding", "officeCode", "eDeliveryConsentedAt", "disregardedEntityChapter4FatcaStatus", "name", "residenceAddress", "residenceCity", "residenceState", "residencePostalCode", "residenceCountry", "mailingAddress", "mailingCity", "mailingState", "mailingPostalCode", "mailingCountry", "disregardedAddress", "disregardedCity", "disregardedState", "disregardedPostalCode", "disregardedCountry", "foreignTin", "ftinNotRequired", "referenceNumber", "giin", "chapter3EntityType", "chapter4FatcaStatus", "disregardedEntity", "disregardedEntityGiin", "benefitLimitation", "part4SponsoringEntity", "part4SponsoringEntityGiin", "part7SponsoringEntity", "part12IgaCountry", "part12IgaType", "part12FatcaStatusUnderIgaAnnexIi", "part12TrusteeName", "part12TrusteeIsForeign", "part12Model2IgaGiin", "box37AExchange", "box37BExchange", "box37BEntity", "part28SponsoringEntity", "part28SponsoringEntityGiin", "signerName", "residenceIsMailing", "citizenshipCountry", "makingTreatyClaim", "treatyCountry", "treatyArticle", "withholdingRate", "incomeType", "treatyReasons", "certifyBox14A", "certifyBox14B", "certifyBox14C", "certifyBox17_1", "certifyBox17_2", "certifyBox18", "certifyBox19", "certifyBox21", "certifyBox22", "certifyBox23", "certifyBox24A", "certifyBox24B", "certifyBox24C", "certifyBox24D", "certifyBox25A", "certifyBox25B", "certifyBox25C", "certifyBox26", "certifyBox27", "certifyBox28A", "certifyBox28B", "certifyBox29A", "certifyBox29B", "certifyBox29C", "certifyBox29D", "certifyBox29E", "certifyBox29F", "certifyBox30", "certifyBox31", "certifyBox32", "certifyBox33", "certifyBox34", "certifyBox35", "certifyBox36", "certifyBox37A", "certifyBox37B", "certifyBox38", "certifyBox39", "certifyBox40A", "certifyBox40B", "certifyBox40C", "certifyBox41", "certifyBox43", "certifyPart29Signature", "part19FormationOrResolutionDate", "part20FilingDate", "part21DeterminationDate", "substantialUsOwners", "birthday", "foreignTinNotRequired", "archived", "referenceId", "companyId", "displayName", "email", "signerCapacity", "updatedAt", "ein", "einType", "certifyBox14", "certifyBox15A", "certifyBox15B", "certifyBox15C", "certifyBox15D", "certifyBox15E", "certifyBox15F", "certifyBox15G", "certifyBox15H", "certifyBox15I", "certifyBox16A", "box16BQddCorporate", "box16BQddPartnership", "box16BQddDisregardedEntity", "certifyBox17A", "certifyBox17B", "certifyBox17C", "certifyBox17D", "certifyBox17E", "certifyBox18A", "certifyBox18B", "certifyBox18C", "certifyBox18D", "certifyBox18E", "certifyBox18F", "certifyBox19A", "certifyBox19B", "certifyBox19C", "certifyBox19D", "certifyBox19E", "certifyBox19F", "certifyBox20", "certifyBox21A", "certifyBox21B", "certifyBox21C", "certifyBox21D", "certifyBox21E", "certifyBox21F", "box23ANameSponsoringEntity", "certifyBox23B", "certifyBox23C", "certifyBox25", "box27ANameSponsoringEntity", "certifyBox27B", "certifyBox28", "certifyBox29", "certifyBox30A", "certifyBox30B", "certifyBox30C", "box32IgaCountry", "box32IgaType", "box32IgaTreatedAs", "box32TrusteeOrSponsor", "box32TrusteeIsForeign", "certifyBox33A", "certifyBox33B", "certifyBox33C", "certifyBox33D", "certifyBox33E", "certifyBox33F", "box37ASecuritiesMarket", "box37BNameOfEntity", "box37BSecuritiesMarket", "certifyBox40", "box41SponsoringEntity", "certifyBox42", "box35FormedOnDate", "box36FiledOnDate", "tinMatchStatus", "signature", "businessClassification", "businessName", "businessOther", "exemptPayeeCode", "exemptFatcaCode", "accountNumber", "foreignCountryIndicator", "foreignAddress", "backupWithholding", "is1099able", "foreignPartnerOwnerOrBeneficiary"]

    @field_validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['w9', 'w4', 'w8imy', 'w8ben', 'w8bene']):
            raise ValueError("must be one of enum values ('w9', 'w4', 'w8imy', 'w8ben', 'w8bene')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of W8BenFormDataModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "type",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in substantial_us_owners (list)
        _items = []
        if self.substantial_us_owners:
            for _item in self.substantial_us_owners:
                if _item:
                    _items.append(_item.to_dict())
            _dict['substantialUsOwners'] = _items
        # set to None if signed_date (nullable) is None
        # and model_fields_set contains the field
        if self.signed_date is None and "signed_date" in self.model_fields_set:
            _dict['signedDate'] = None

        # set to None if created_at (nullable) is None
        # and model_fields_set contains the field
        if self.created_at is None and "created_at" in self.model_fields_set:
            _dict['createdAt'] = None

        # set to None if employee_first_name (nullable) is None
        # and model_fields_set contains the field
        if self.employee_first_name is None and "employee_first_name" in self.model_fields_set:
            _dict['employeeFirstName'] = None

        # set to None if employee_middle_name (nullable) is None
        # and model_fields_set contains the field
        if self.employee_middle_name is None and "employee_middle_name" in self.model_fields_set:
            _dict['employeeMiddleName'] = None

        # set to None if employee_last_name (nullable) is None
        # and model_fields_set contains the field
        if self.employee_last_name is None and "employee_last_name" in self.model_fields_set:
            _dict['employeeLastName'] = None

        # set to None if employee_name_suffix (nullable) is None
        # and model_fields_set contains the field
        if self.employee_name_suffix is None and "employee_name_suffix" in self.model_fields_set:
            _dict['employeeNameSuffix'] = None

        # set to None if address (nullable) is None
        # and model_fields_set contains the field
        if self.address is None and "address" in self.model_fields_set:
            _dict['address'] = None

        # set to None if city (nullable) is None
        # and model_fields_set contains the field
        if self.city is None and "city" in self.model_fields_set:
            _dict['city'] = None

        # set to None if state (nullable) is None
        # and model_fields_set contains the field
        if self.state is None and "state" in self.model_fields_set:
            _dict['state'] = None

        # set to None if zip (nullable) is None
        # and model_fields_set contains the field
        if self.zip is None and "zip" in self.model_fields_set:
            _dict['zip'] = None

        # set to None if type_of_tin (nullable) is None
        # and model_fields_set contains the field
        if self.type_of_tin is None and "type_of_tin" in self.model_fields_set:
            _dict['typeOfTin'] = None

        # set to None if tin (nullable) is None
        # and model_fields_set contains the field
        if self.tin is None and "tin" in self.model_fields_set:
            _dict['tin'] = None

        # set to None if box3_marital_status (nullable) is None
        # and model_fields_set contains the field
        if self.box3_marital_status is None and "box3_marital_status" in self.model_fields_set:
            _dict['box3MaritalStatus'] = None

        # set to None if box4_last_name_differs (nullable) is None
        # and model_fields_set contains the field
        if self.box4_last_name_differs is None and "box4_last_name_differs" in self.model_fields_set:
            _dict['box4LastNameDiffers'] = None

        # set to None if box5_num_allowances (nullable) is None
        # and model_fields_set contains the field
        if self.box5_num_allowances is None and "box5_num_allowances" in self.model_fields_set:
            _dict['box5NumAllowances'] = None

        # set to None if other_dependents (nullable) is None
        # and model_fields_set contains the field
        if self.other_dependents is None and "other_dependents" in self.model_fields_set:
            _dict['otherDependents'] = None

        # set to None if non_job_income (nullable) is None
        # and model_fields_set contains the field
        if self.non_job_income is None and "non_job_income" in self.model_fields_set:
            _dict['nonJobIncome'] = None

        # set to None if deductions (nullable) is None
        # and model_fields_set contains the field
        if self.deductions is None and "deductions" in self.model_fields_set:
            _dict['deductions'] = None

        # set to None if box6_additional_withheld (nullable) is None
        # and model_fields_set contains the field
        if self.box6_additional_withheld is None and "box6_additional_withheld" in self.model_fields_set:
            _dict['box6AdditionalWithheld'] = None

        # set to None if box7_exempt_from_withholding (nullable) is None
        # and model_fields_set contains the field
        if self.box7_exempt_from_withholding is None and "box7_exempt_from_withholding" in self.model_fields_set:
            _dict['box7ExemptFromWithholding'] = None

        # set to None if office_code (nullable) is None
        # and model_fields_set contains the field
        if self.office_code is None and "office_code" in self.model_fields_set:
            _dict['officeCode'] = None

        # set to None if e_delivery_consented_at (nullable) is None
        # and model_fields_set contains the field
        if self.e_delivery_consented_at is None and "e_delivery_consented_at" in self.model_fields_set:
            _dict['eDeliveryConsentedAt'] = None

        # set to None if disregarded_entity_chapter4_fatca_status (nullable) is None
        # and model_fields_set contains the field
        if self.disregarded_entity_chapter4_fatca_status is None and "disregarded_entity_chapter4_fatca_status" in self.model_fields_set:
            _dict['disregardedEntityChapter4FatcaStatus'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['name'] = None

        # set to None if residence_address (nullable) is None
        # and model_fields_set contains the field
        if self.residence_address is None and "residence_address" in self.model_fields_set:
            _dict['residenceAddress'] = None

        # set to None if residence_city (nullable) is None
        # and model_fields_set contains the field
        if self.residence_city is None and "residence_city" in self.model_fields_set:
            _dict['residenceCity'] = None

        # set to None if residence_state (nullable) is None
        # and model_fields_set contains the field
        if self.residence_state is None and "residence_state" in self.model_fields_set:
            _dict['residenceState'] = None

        # set to None if residence_postal_code (nullable) is None
        # and model_fields_set contains the field
        if self.residence_postal_code is None and "residence_postal_code" in self.model_fields_set:
            _dict['residencePostalCode'] = None

        # set to None if residence_country (nullable) is None
        # and model_fields_set contains the field
        if self.residence_country is None and "residence_country" in self.model_fields_set:
            _dict['residenceCountry'] = None

        # set to None if mailing_address (nullable) is None
        # and model_fields_set contains the field
        if self.mailing_address is None and "mailing_address" in self.model_fields_set:
            _dict['mailingAddress'] = None

        # set to None if mailing_city (nullable) is None
        # and model_fields_set contains the field
        if self.mailing_city is None and "mailing_city" in self.model_fields_set:
            _dict['mailingCity'] = None

        # set to None if mailing_state (nullable) is None
        # and model_fields_set contains the field
        if self.mailing_state is None and "mailing_state" in self.model_fields_set:
            _dict['mailingState'] = None

        # set to None if mailing_postal_code (nullable) is None
        # and model_fields_set contains the field
        if self.mailing_postal_code is None and "mailing_postal_code" in self.model_fields_set:
            _dict['mailingPostalCode'] = None

        # set to None if mailing_country (nullable) is None
        # and model_fields_set contains the field
        if self.mailing_country is None and "mailing_country" in self.model_fields_set:
            _dict['mailingCountry'] = None

        # set to None if disregarded_address (nullable) is None
        # and model_fields_set contains the field
        if self.disregarded_address is None and "disregarded_address" in self.model_fields_set:
            _dict['disregardedAddress'] = None

        # set to None if disregarded_city (nullable) is None
        # and model_fields_set contains the field
        if self.disregarded_city is None and "disregarded_city" in self.model_fields_set:
            _dict['disregardedCity'] = None

        # set to None if disregarded_state (nullable) is None
        # and model_fields_set contains the field
        if self.disregarded_state is None and "disregarded_state" in self.model_fields_set:
            _dict['disregardedState'] = None

        # set to None if disregarded_postal_code (nullable) is None
        # and model_fields_set contains the field
        if self.disregarded_postal_code is None and "disregarded_postal_code" in self.model_fields_set:
            _dict['disregardedPostalCode'] = None

        # set to None if disregarded_country (nullable) is None
        # and model_fields_set contains the field
        if self.disregarded_country is None and "disregarded_country" in self.model_fields_set:
            _dict['disregardedCountry'] = None

        # set to None if foreign_tin (nullable) is None
        # and model_fields_set contains the field
        if self.foreign_tin is None and "foreign_tin" in self.model_fields_set:
            _dict['foreignTin'] = None

        # set to None if ftin_not_required (nullable) is None
        # and model_fields_set contains the field
        if self.ftin_not_required is None and "ftin_not_required" in self.model_fields_set:
            _dict['ftinNotRequired'] = None

        # set to None if reference_number (nullable) is None
        # and model_fields_set contains the field
        if self.reference_number is None and "reference_number" in self.model_fields_set:
            _dict['referenceNumber'] = None

        # set to None if giin (nullable) is None
        # and model_fields_set contains the field
        if self.giin is None and "giin" in self.model_fields_set:
            _dict['giin'] = None

        # set to None if chapter3_entity_type (nullable) is None
        # and model_fields_set contains the field
        if self.chapter3_entity_type is None and "chapter3_entity_type" in self.model_fields_set:
            _dict['chapter3EntityType'] = None

        # set to None if chapter4_fatca_status (nullable) is None
        # and model_fields_set contains the field
        if self.chapter4_fatca_status is None and "chapter4_fatca_status" in self.model_fields_set:
            _dict['chapter4FatcaStatus'] = None

        # set to None if disregarded_entity (nullable) is None
        # and model_fields_set contains the field
        if self.disregarded_entity is None and "disregarded_entity" in self.model_fields_set:
            _dict['disregardedEntity'] = None

        # set to None if disregarded_entity_giin (nullable) is None
        # and model_fields_set contains the field
        if self.disregarded_entity_giin is None and "disregarded_entity_giin" in self.model_fields_set:
            _dict['disregardedEntityGiin'] = None

        # set to None if benefit_limitation (nullable) is None
        # and model_fields_set contains the field
        if self.benefit_limitation is None and "benefit_limitation" in self.model_fields_set:
            _dict['benefitLimitation'] = None

        # set to None if part4_sponsoring_entity (nullable) is None
        # and model_fields_set contains the field
        if self.part4_sponsoring_entity is None and "part4_sponsoring_entity" in self.model_fields_set:
            _dict['part4SponsoringEntity'] = None

        # set to None if part4_sponsoring_entity_giin (nullable) is None
        # and model_fields_set contains the field
        if self.part4_sponsoring_entity_giin is None and "part4_sponsoring_entity_giin" in self.model_fields_set:
            _dict['part4SponsoringEntityGiin'] = None

        # set to None if part7_sponsoring_entity (nullable) is None
        # and model_fields_set contains the field
        if self.part7_sponsoring_entity is None and "part7_sponsoring_entity" in self.model_fields_set:
            _dict['part7SponsoringEntity'] = None

        # set to None if part12_iga_country (nullable) is None
        # and model_fields_set contains the field
        if self.part12_iga_country is None and "part12_iga_country" in self.model_fields_set:
            _dict['part12IgaCountry'] = None

        # set to None if part12_iga_type (nullable) is None
        # and model_fields_set contains the field
        if self.part12_iga_type is None and "part12_iga_type" in self.model_fields_set:
            _dict['part12IgaType'] = None

        # set to None if part12_fatca_status_under_iga_annex_ii (nullable) is None
        # and model_fields_set contains the field
        if self.part12_fatca_status_under_iga_annex_ii is None and "part12_fatca_status_under_iga_annex_ii" in self.model_fields_set:
            _dict['part12FatcaStatusUnderIgaAnnexIi'] = None

        # set to None if part12_trustee_name (nullable) is None
        # and model_fields_set contains the field
        if self.part12_trustee_name is None and "part12_trustee_name" in self.model_fields_set:
            _dict['part12TrusteeName'] = None

        # set to None if part12_trustee_is_foreign (nullable) is None
        # and model_fields_set contains the field
        if self.part12_trustee_is_foreign is None and "part12_trustee_is_foreign" in self.model_fields_set:
            _dict['part12TrusteeIsForeign'] = None

        # set to None if part12_model2_iga_giin (nullable) is None
        # and model_fields_set contains the field
        if self.part12_model2_iga_giin is None and "part12_model2_iga_giin" in self.model_fields_set:
            _dict['part12Model2IgaGiin'] = None

        # set to None if box37_a_exchange (nullable) is None
        # and model_fields_set contains the field
        if self.box37_a_exchange is None and "box37_a_exchange" in self.model_fields_set:
            _dict['box37AExchange'] = None

        # set to None if box37_b_exchange (nullable) is None
        # and model_fields_set contains the field
        if self.box37_b_exchange is None and "box37_b_exchange" in self.model_fields_set:
            _dict['box37BExchange'] = None

        # set to None if box37_b_entity (nullable) is None
        # and model_fields_set contains the field
        if self.box37_b_entity is None and "box37_b_entity" in self.model_fields_set:
            _dict['box37BEntity'] = None

        # set to None if part28_sponsoring_entity (nullable) is None
        # and model_fields_set contains the field
        if self.part28_sponsoring_entity is None and "part28_sponsoring_entity" in self.model_fields_set:
            _dict['part28SponsoringEntity'] = None

        # set to None if part28_sponsoring_entity_giin (nullable) is None
        # and model_fields_set contains the field
        if self.part28_sponsoring_entity_giin is None and "part28_sponsoring_entity_giin" in self.model_fields_set:
            _dict['part28SponsoringEntityGiin'] = None

        # set to None if signer_name (nullable) is None
        # and model_fields_set contains the field
        if self.signer_name is None and "signer_name" in self.model_fields_set:
            _dict['signerName'] = None

        # set to None if citizenship_country (nullable) is None
        # and model_fields_set contains the field
        if self.citizenship_country is None and "citizenship_country" in self.model_fields_set:
            _dict['citizenshipCountry'] = None

        # set to None if making_treaty_claim (nullable) is None
        # and model_fields_set contains the field
        if self.making_treaty_claim is None and "making_treaty_claim" in self.model_fields_set:
            _dict['makingTreatyClaim'] = None

        # set to None if treaty_country (nullable) is None
        # and model_fields_set contains the field
        if self.treaty_country is None and "treaty_country" in self.model_fields_set:
            _dict['treatyCountry'] = None

        # set to None if treaty_article (nullable) is None
        # and model_fields_set contains the field
        if self.treaty_article is None and "treaty_article" in self.model_fields_set:
            _dict['treatyArticle'] = None

        # set to None if withholding_rate (nullable) is None
        # and model_fields_set contains the field
        if self.withholding_rate is None and "withholding_rate" in self.model_fields_set:
            _dict['withholdingRate'] = None

        # set to None if income_type (nullable) is None
        # and model_fields_set contains the field
        if self.income_type is None and "income_type" in self.model_fields_set:
            _dict['incomeType'] = None

        # set to None if treaty_reasons (nullable) is None
        # and model_fields_set contains the field
        if self.treaty_reasons is None and "treaty_reasons" in self.model_fields_set:
            _dict['treatyReasons'] = None

        # set to None if part19_formation_or_resolution_date (nullable) is None
        # and model_fields_set contains the field
        if self.part19_formation_or_resolution_date is None and "part19_formation_or_resolution_date" in self.model_fields_set:
            _dict['part19FormationOrResolutionDate'] = None

        # set to None if part20_filing_date (nullable) is None
        # and model_fields_set contains the field
        if self.part20_filing_date is None and "part20_filing_date" in self.model_fields_set:
            _dict['part20FilingDate'] = None

        # set to None if part21_determination_date (nullable) is None
        # and model_fields_set contains the field
        if self.part21_determination_date is None and "part21_determination_date" in self.model_fields_set:
            _dict['part21DeterminationDate'] = None

        # set to None if birthday (nullable) is None
        # and model_fields_set contains the field
        if self.birthday is None and "birthday" in self.model_fields_set:
            _dict['birthday'] = None

        # set to None if foreign_tin_not_required (nullable) is None
        # and model_fields_set contains the field
        if self.foreign_tin_not_required is None and "foreign_tin_not_required" in self.model_fields_set:
            _dict['foreignTinNotRequired'] = None

        # set to None if archived (nullable) is None
        # and model_fields_set contains the field
        if self.archived is None and "archived" in self.model_fields_set:
            _dict['archived'] = None

        # set to None if reference_id (nullable) is None
        # and model_fields_set contains the field
        if self.reference_id is None and "reference_id" in self.model_fields_set:
            _dict['referenceId'] = None

        # set to None if company_id (nullable) is None
        # and model_fields_set contains the field
        if self.company_id is None and "company_id" in self.model_fields_set:
            _dict['companyId'] = None

        # set to None if display_name (nullable) is None
        # and model_fields_set contains the field
        if self.display_name is None and "display_name" in self.model_fields_set:
            _dict['displayName'] = None

        # set to None if email (nullable) is None
        # and model_fields_set contains the field
        if self.email is None and "email" in self.model_fields_set:
            _dict['email'] = None

        # set to None if signer_capacity (nullable) is None
        # and model_fields_set contains the field
        if self.signer_capacity is None and "signer_capacity" in self.model_fields_set:
            _dict['signerCapacity'] = None

        # set to None if updated_at (nullable) is None
        # and model_fields_set contains the field
        if self.updated_at is None and "updated_at" in self.model_fields_set:
            _dict['updatedAt'] = None

        # set to None if ein (nullable) is None
        # and model_fields_set contains the field
        if self.ein is None and "ein" in self.model_fields_set:
            _dict['ein'] = None

        # set to None if ein_type (nullable) is None
        # and model_fields_set contains the field
        if self.ein_type is None and "ein_type" in self.model_fields_set:
            _dict['einType'] = None

        # set to None if box23_a_name_sponsoring_entity (nullable) is None
        # and model_fields_set contains the field
        if self.box23_a_name_sponsoring_entity is None and "box23_a_name_sponsoring_entity" in self.model_fields_set:
            _dict['box23ANameSponsoringEntity'] = None

        # set to None if box27_a_name_sponsoring_entity (nullable) is None
        # and model_fields_set contains the field
        if self.box27_a_name_sponsoring_entity is None and "box27_a_name_sponsoring_entity" in self.model_fields_set:
            _dict['box27ANameSponsoringEntity'] = None

        # set to None if box32_iga_country (nullable) is None
        # and model_fields_set contains the field
        if self.box32_iga_country is None and "box32_iga_country" in self.model_fields_set:
            _dict['box32IgaCountry'] = None

        # set to None if box32_iga_type (nullable) is None
        # and model_fields_set contains the field
        if self.box32_iga_type is None and "box32_iga_type" in self.model_fields_set:
            _dict['box32IgaType'] = None

        # set to None if box32_iga_treated_as (nullable) is None
        # and model_fields_set contains the field
        if self.box32_iga_treated_as is None and "box32_iga_treated_as" in self.model_fields_set:
            _dict['box32IgaTreatedAs'] = None

        # set to None if box32_trustee_or_sponsor (nullable) is None
        # and model_fields_set contains the field
        if self.box32_trustee_or_sponsor is None and "box32_trustee_or_sponsor" in self.model_fields_set:
            _dict['box32TrusteeOrSponsor'] = None

        # set to None if box37_a_securities_market (nullable) is None
        # and model_fields_set contains the field
        if self.box37_a_securities_market is None and "box37_a_securities_market" in self.model_fields_set:
            _dict['box37ASecuritiesMarket'] = None

        # set to None if box37_b_name_of_entity (nullable) is None
        # and model_fields_set contains the field
        if self.box37_b_name_of_entity is None and "box37_b_name_of_entity" in self.model_fields_set:
            _dict['box37BNameOfEntity'] = None

        # set to None if box37_b_securities_market (nullable) is None
        # and model_fields_set contains the field
        if self.box37_b_securities_market is None and "box37_b_securities_market" in self.model_fields_set:
            _dict['box37BSecuritiesMarket'] = None

        # set to None if box41_sponsoring_entity (nullable) is None
        # and model_fields_set contains the field
        if self.box41_sponsoring_entity is None and "box41_sponsoring_entity" in self.model_fields_set:
            _dict['box41SponsoringEntity'] = None

        # set to None if box35_formed_on_date (nullable) is None
        # and model_fields_set contains the field
        if self.box35_formed_on_date is None and "box35_formed_on_date" in self.model_fields_set:
            _dict['box35FormedOnDate'] = None

        # set to None if box36_filed_on_date (nullable) is None
        # and model_fields_set contains the field
        if self.box36_filed_on_date is None and "box36_filed_on_date" in self.model_fields_set:
            _dict['box36FiledOnDate'] = None

        # set to None if tin_match_status (nullable) is None
        # and model_fields_set contains the field
        if self.tin_match_status is None and "tin_match_status" in self.model_fields_set:
            _dict['tinMatchStatus'] = None

        # set to None if signature (nullable) is None
        # and model_fields_set contains the field
        if self.signature is None and "signature" in self.model_fields_set:
            _dict['signature'] = None

        # set to None if business_classification (nullable) is None
        # and model_fields_set contains the field
        if self.business_classification is None and "business_classification" in self.model_fields_set:
            _dict['businessClassification'] = None

        # set to None if business_name (nullable) is None
        # and model_fields_set contains the field
        if self.business_name is None and "business_name" in self.model_fields_set:
            _dict['businessName'] = None

        # set to None if business_other (nullable) is None
        # and model_fields_set contains the field
        if self.business_other is None and "business_other" in self.model_fields_set:
            _dict['businessOther'] = None

        # set to None if exempt_payee_code (nullable) is None
        # and model_fields_set contains the field
        if self.exempt_payee_code is None and "exempt_payee_code" in self.model_fields_set:
            _dict['exemptPayeeCode'] = None

        # set to None if exempt_fatca_code (nullable) is None
        # and model_fields_set contains the field
        if self.exempt_fatca_code is None and "exempt_fatca_code" in self.model_fields_set:
            _dict['exemptFatcaCode'] = None

        # set to None if account_number (nullable) is None
        # and model_fields_set contains the field
        if self.account_number is None and "account_number" in self.model_fields_set:
            _dict['accountNumber'] = None

        # set to None if foreign_country_indicator (nullable) is None
        # and model_fields_set contains the field
        if self.foreign_country_indicator is None and "foreign_country_indicator" in self.model_fields_set:
            _dict['foreignCountryIndicator'] = None

        # set to None if foreign_address (nullable) is None
        # and model_fields_set contains the field
        if self.foreign_address is None and "foreign_address" in self.model_fields_set:
            _dict['foreignAddress'] = None

        # set to None if backup_withholding (nullable) is None
        # and model_fields_set contains the field
        if self.backup_withholding is None and "backup_withholding" in self.model_fields_set:
            _dict['backupWithholding'] = None

        # set to None if is1099able (nullable) is None
        # and model_fields_set contains the field
        if self.is1099able is None and "is1099able" in self.model_fields_set:
            _dict['is1099able'] = None

        # set to None if foreign_partner_owner_or_beneficiary (nullable) is None
        # and model_fields_set contains the field
        if self.foreign_partner_owner_or_beneficiary is None and "foreign_partner_owner_or_beneficiary" in self.model_fields_set:
            _dict['foreignPartnerOwnerOrBeneficiary'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of W8BenFormDataModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "type": obj.get("type"),
            "signedDate": obj.get("signedDate"),
            "createdAt": obj.get("createdAt"),
            "employeeFirstName": obj.get("employeeFirstName"),
            "employeeMiddleName": obj.get("employeeMiddleName"),
            "employeeLastName": obj.get("employeeLastName"),
            "employeeNameSuffix": obj.get("employeeNameSuffix"),
            "address": obj.get("address"),
            "city": obj.get("city"),
            "state": obj.get("state"),
            "zip": obj.get("zip"),
            "typeOfTin": obj.get("typeOfTin"),
            "tin": obj.get("tin"),
            "box3MaritalStatus": obj.get("box3MaritalStatus"),
            "box4LastNameDiffers": obj.get("box4LastNameDiffers"),
            "box5NumAllowances": obj.get("box5NumAllowances"),
            "otherDependents": obj.get("otherDependents"),
            "nonJobIncome": obj.get("nonJobIncome"),
            "deductions": obj.get("deductions"),
            "box6AdditionalWithheld": obj.get("box6AdditionalWithheld"),
            "box7ExemptFromWithholding": obj.get("box7ExemptFromWithholding"),
            "officeCode": obj.get("officeCode"),
            "eDeliveryConsentedAt": obj.get("eDeliveryConsentedAt"),
            "disregardedEntityChapter4FatcaStatus": obj.get("disregardedEntityChapter4FatcaStatus"),
            "name": obj.get("name"),
            "residenceAddress": obj.get("residenceAddress"),
            "residenceCity": obj.get("residenceCity"),
            "residenceState": obj.get("residenceState"),
            "residencePostalCode": obj.get("residencePostalCode"),
            "residenceCountry": obj.get("residenceCountry"),
            "mailingAddress": obj.get("mailingAddress"),
            "mailingCity": obj.get("mailingCity"),
            "mailingState": obj.get("mailingState"),
            "mailingPostalCode": obj.get("mailingPostalCode"),
            "mailingCountry": obj.get("mailingCountry"),
            "disregardedAddress": obj.get("disregardedAddress"),
            "disregardedCity": obj.get("disregardedCity"),
            "disregardedState": obj.get("disregardedState"),
            "disregardedPostalCode": obj.get("disregardedPostalCode"),
            "disregardedCountry": obj.get("disregardedCountry"),
            "foreignTin": obj.get("foreignTin"),
            "ftinNotRequired": obj.get("ftinNotRequired"),
            "referenceNumber": obj.get("referenceNumber"),
            "giin": obj.get("giin"),
            "chapter3EntityType": obj.get("chapter3EntityType"),
            "chapter4FatcaStatus": obj.get("chapter4FatcaStatus"),
            "disregardedEntity": obj.get("disregardedEntity"),
            "disregardedEntityGiin": obj.get("disregardedEntityGiin"),
            "benefitLimitation": obj.get("benefitLimitation"),
            "part4SponsoringEntity": obj.get("part4SponsoringEntity"),
            "part4SponsoringEntityGiin": obj.get("part4SponsoringEntityGiin"),
            "part7SponsoringEntity": obj.get("part7SponsoringEntity"),
            "part12IgaCountry": obj.get("part12IgaCountry"),
            "part12IgaType": obj.get("part12IgaType"),
            "part12FatcaStatusUnderIgaAnnexIi": obj.get("part12FatcaStatusUnderIgaAnnexIi"),
            "part12TrusteeName": obj.get("part12TrusteeName"),
            "part12TrusteeIsForeign": obj.get("part12TrusteeIsForeign"),
            "part12Model2IgaGiin": obj.get("part12Model2IgaGiin"),
            "box37AExchange": obj.get("box37AExchange"),
            "box37BExchange": obj.get("box37BExchange"),
            "box37BEntity": obj.get("box37BEntity"),
            "part28SponsoringEntity": obj.get("part28SponsoringEntity"),
            "part28SponsoringEntityGiin": obj.get("part28SponsoringEntityGiin"),
            "signerName": obj.get("signerName"),
            "residenceIsMailing": obj.get("residenceIsMailing"),
            "citizenshipCountry": obj.get("citizenshipCountry"),
            "makingTreatyClaim": obj.get("makingTreatyClaim"),
            "treatyCountry": obj.get("treatyCountry"),
            "treatyArticle": obj.get("treatyArticle"),
            "withholdingRate": obj.get("withholdingRate"),
            "incomeType": obj.get("incomeType"),
            "treatyReasons": obj.get("treatyReasons"),
            "certifyBox14A": obj.get("certifyBox14A"),
            "certifyBox14B": obj.get("certifyBox14B"),
            "certifyBox14C": obj.get("certifyBox14C"),
            "certifyBox17_1": obj.get("certifyBox17_1"),
            "certifyBox17_2": obj.get("certifyBox17_2"),
            "certifyBox18": obj.get("certifyBox18"),
            "certifyBox19": obj.get("certifyBox19"),
            "certifyBox21": obj.get("certifyBox21"),
            "certifyBox22": obj.get("certifyBox22"),
            "certifyBox23": obj.get("certifyBox23"),
            "certifyBox24A": obj.get("certifyBox24A"),
            "certifyBox24B": obj.get("certifyBox24B"),
            "certifyBox24C": obj.get("certifyBox24C"),
            "certifyBox24D": obj.get("certifyBox24D"),
            "certifyBox25A": obj.get("certifyBox25A"),
            "certifyBox25B": obj.get("certifyBox25B"),
            "certifyBox25C": obj.get("certifyBox25C"),
            "certifyBox26": obj.get("certifyBox26"),
            "certifyBox27": obj.get("certifyBox27"),
            "certifyBox28A": obj.get("certifyBox28A"),
            "certifyBox28B": obj.get("certifyBox28B"),
            "certifyBox29A": obj.get("certifyBox29A"),
            "certifyBox29B": obj.get("certifyBox29B"),
            "certifyBox29C": obj.get("certifyBox29C"),
            "certifyBox29D": obj.get("certifyBox29D"),
            "certifyBox29E": obj.get("certifyBox29E"),
            "certifyBox29F": obj.get("certifyBox29F"),
            "certifyBox30": obj.get("certifyBox30"),
            "certifyBox31": obj.get("certifyBox31"),
            "certifyBox32": obj.get("certifyBox32"),
            "certifyBox33": obj.get("certifyBox33"),
            "certifyBox34": obj.get("certifyBox34"),
            "certifyBox35": obj.get("certifyBox35"),
            "certifyBox36": obj.get("certifyBox36"),
            "certifyBox37A": obj.get("certifyBox37A"),
            "certifyBox37B": obj.get("certifyBox37B"),
            "certifyBox38": obj.get("certifyBox38"),
            "certifyBox39": obj.get("certifyBox39"),
            "certifyBox40A": obj.get("certifyBox40A"),
            "certifyBox40B": obj.get("certifyBox40B"),
            "certifyBox40C": obj.get("certifyBox40C"),
            "certifyBox41": obj.get("certifyBox41"),
            "certifyBox43": obj.get("certifyBox43"),
            "certifyPart29Signature": obj.get("certifyPart29Signature"),
            "part19FormationOrResolutionDate": obj.get("part19FormationOrResolutionDate"),
            "part20FilingDate": obj.get("part20FilingDate"),
            "part21DeterminationDate": obj.get("part21DeterminationDate"),
            "substantialUsOwners": [W8BenESubstantialUsOwnerDataModel.from_dict(_item) for _item in obj["substantialUsOwners"]] if obj.get("substantialUsOwners") is not None else None,
            "birthday": obj.get("birthday"),
            "foreignTinNotRequired": obj.get("foreignTinNotRequired"),
            "archived": obj.get("archived"),
            "referenceId": obj.get("referenceId"),
            "companyId": obj.get("companyId"),
            "displayName": obj.get("displayName"),
            "email": obj.get("email"),
            "signerCapacity": obj.get("signerCapacity"),
            "updatedAt": obj.get("updatedAt"),
            "ein": obj.get("ein"),
            "einType": obj.get("einType"),
            "certifyBox14": obj.get("certifyBox14"),
            "certifyBox15A": obj.get("certifyBox15A"),
            "certifyBox15B": obj.get("certifyBox15B"),
            "certifyBox15C": obj.get("certifyBox15C"),
            "certifyBox15D": obj.get("certifyBox15D"),
            "certifyBox15E": obj.get("certifyBox15E"),
            "certifyBox15F": obj.get("certifyBox15F"),
            "certifyBox15G": obj.get("certifyBox15G"),
            "certifyBox15H": obj.get("certifyBox15H"),
            "certifyBox15I": obj.get("certifyBox15I"),
            "certifyBox16A": obj.get("certifyBox16A"),
            "box16BQddCorporate": obj.get("box16BQddCorporate"),
            "box16BQddPartnership": obj.get("box16BQddPartnership"),
            "box16BQddDisregardedEntity": obj.get("box16BQddDisregardedEntity"),
            "certifyBox17A": obj.get("certifyBox17A"),
            "certifyBox17B": obj.get("certifyBox17B"),
            "certifyBox17C": obj.get("certifyBox17C"),
            "certifyBox17D": obj.get("certifyBox17D"),
            "certifyBox17E": obj.get("certifyBox17E"),
            "certifyBox18A": obj.get("certifyBox18A"),
            "certifyBox18B": obj.get("certifyBox18B"),
            "certifyBox18C": obj.get("certifyBox18C"),
            "certifyBox18D": obj.get("certifyBox18D"),
            "certifyBox18E": obj.get("certifyBox18E"),
            "certifyBox18F": obj.get("certifyBox18F"),
            "certifyBox19A": obj.get("certifyBox19A"),
            "certifyBox19B": obj.get("certifyBox19B"),
            "certifyBox19C": obj.get("certifyBox19C"),
            "certifyBox19D": obj.get("certifyBox19D"),
            "certifyBox19E": obj.get("certifyBox19E"),
            "certifyBox19F": obj.get("certifyBox19F"),
            "certifyBox20": obj.get("certifyBox20"),
            "certifyBox21A": obj.get("certifyBox21A"),
            "certifyBox21B": obj.get("certifyBox21B"),
            "certifyBox21C": obj.get("certifyBox21C"),
            "certifyBox21D": obj.get("certifyBox21D"),
            "certifyBox21E": obj.get("certifyBox21E"),
            "certifyBox21F": obj.get("certifyBox21F"),
            "box23ANameSponsoringEntity": obj.get("box23ANameSponsoringEntity"),
            "certifyBox23B": obj.get("certifyBox23B"),
            "certifyBox23C": obj.get("certifyBox23C"),
            "certifyBox25": obj.get("certifyBox25"),
            "box27ANameSponsoringEntity": obj.get("box27ANameSponsoringEntity"),
            "certifyBox27B": obj.get("certifyBox27B"),
            "certifyBox28": obj.get("certifyBox28"),
            "certifyBox29": obj.get("certifyBox29"),
            "certifyBox30A": obj.get("certifyBox30A"),
            "certifyBox30B": obj.get("certifyBox30B"),
            "certifyBox30C": obj.get("certifyBox30C"),
            "box32IgaCountry": obj.get("box32IgaCountry"),
            "box32IgaType": obj.get("box32IgaType"),
            "box32IgaTreatedAs": obj.get("box32IgaTreatedAs"),
            "box32TrusteeOrSponsor": obj.get("box32TrusteeOrSponsor"),
            "box32TrusteeIsForeign": obj.get("box32TrusteeIsForeign"),
            "certifyBox33A": obj.get("certifyBox33A"),
            "certifyBox33B": obj.get("certifyBox33B"),
            "certifyBox33C": obj.get("certifyBox33C"),
            "certifyBox33D": obj.get("certifyBox33D"),
            "certifyBox33E": obj.get("certifyBox33E"),
            "certifyBox33F": obj.get("certifyBox33F"),
            "box37ASecuritiesMarket": obj.get("box37ASecuritiesMarket"),
            "box37BNameOfEntity": obj.get("box37BNameOfEntity"),
            "box37BSecuritiesMarket": obj.get("box37BSecuritiesMarket"),
            "certifyBox40": obj.get("certifyBox40"),
            "box41SponsoringEntity": obj.get("box41SponsoringEntity"),
            "certifyBox42": obj.get("certifyBox42"),
            "box35FormedOnDate": obj.get("box35FormedOnDate"),
            "box36FiledOnDate": obj.get("box36FiledOnDate"),
            "tinMatchStatus": obj.get("tinMatchStatus"),
            "signature": obj.get("signature"),
            "businessClassification": obj.get("businessClassification"),
            "businessName": obj.get("businessName"),
            "businessOther": obj.get("businessOther"),
            "exemptPayeeCode": obj.get("exemptPayeeCode"),
            "exemptFatcaCode": obj.get("exemptFatcaCode"),
            "accountNumber": obj.get("accountNumber"),
            "foreignCountryIndicator": obj.get("foreignCountryIndicator"),
            "foreignAddress": obj.get("foreignAddress"),
            "backupWithholding": obj.get("backupWithholding"),
            "is1099able": obj.get("is1099able"),
            "foreignPartnerOwnerOrBeneficiary": obj.get("foreignPartnerOwnerOrBeneficiary")
        })
        return _obj





from datetime import datetime
import json

import urllib3
import urllib

from Avalara.SDK.oauth_helper.AvalaraCache import AvalaraCache
from Avalara.SDK.oauth_helper.AvalaraOauth2Configuration import (
    AvalaraOauth2Configuration,
)
from Avalara.SDK.oauth_helper.AvalaraSdkOauthUtils import (
    AVALARA_SDK_CONSTANTS,
    AvalaraApiEnvironment,
)


class AvalaraOauth2Client:
    """Utility class with Avalara Oauth2 helper methods"""

    client_id: str
    client_secret: str
    required_scopes: str
    avalara_api_environment: AvalaraApiEnvironment
    cache: AvalaraCache

    # constructor
    def __init__(
        self,
        client_id,
        client_secret,
        required_scopes,
        avalara_api_environment,
    ):
        self.client_id = client_id
        self.client_secret = client_secret
        self.required_scopes = required_scopes
        self.oauth2_config = AvalaraOauth2Configuration(
            required_scopes, avalara_api_environment
        )
        self.cache = AvalaraCache()
        self.__is_token_returned_from_cache = False
        self.__token_renewal_seconds_before_ttl_end = 300

    def get_avalara_access_token_info(self) -> dict:
        """Provides Avalara api access token and related data
           If access token information is present in cache it is returned
           from cache. Information is fetched and added to cache if it is
           not present.
           As a business rule ttl for cach expiry is set 5 minutes earlier than
           actual token lifetime.

        Returns:
            dict: Dictionary with Avalara access token and related data
        """
        cached_token_data = self.__get_access_token_data_from_cache()
        if cached_token_data is not None:
            self.__is_token_returned_from_cache = True
            return cached_token_data
        else:
            access_token_data = self.__get_access_token_data(
                self.oauth2_config.avalara_oidc_data.token_endpoint,
                self.client_id,
                self.client_secret,
                grant_type=(
                    AVALARA_SDK_CONSTANTS.AVALARA_OIDC_VALUES_GRANT_TYPE_CLIENT_CREDENTIALS
                ),
            )
            self.__add_access_token_data_in_cache(access_token_data)
            self.__is_token_returned_from_cache = False
            return access_token_data

    def __add_access_token_data_in_cache(self, access_token_data: dict):
        clienr_creds_validity_seconds = access_token_data.get(
            AVALARA_SDK_CONSTANTS.OAUTH_RESPONSE_KEY_CLIENT_CREDS_EXPIRES_IN
        )
        ttl_seconds = (
            int(clienr_creds_validity_seconds)
            if clienr_creds_validity_seconds
            else None
        )
        if ttl_seconds is None:
            ttl_seconds = 3600
        self.cache.set_item_with_ttl(
            self.client_id,
            access_token_data,
            ttl_seconds - self.__token_renewal_seconds_before_ttl_end,
        )

    def __get_access_token_data_from_cache(self):
        cached_item = self.cache.get(self.client_id)
        if (
            cached_item is not None
            and cached_item[0].get("access_token") is not None
        ):
            item_expired = datetime.utcnow() > cached_item[1]
            if item_expired:
                return None
            else:
                return cached_item[0]
        else:
            return None

    def get_access_token_for_device_flow(self, device_code) -> dict:
        '''Returns access token for device flow'''

        return self.__get_access_token_data(
            self.oauth2_config.avalara_oidc_data.token_endpoint,
            self.client_id,
            self.client_secret,
            grant_type=AVALARA_SDK_CONSTANTS.AVALARA_OIDC_VALUES_GRANT_TYPE_DEVICE_FLOW,
            device_code=device_code,
        )

    def initiate_device_authorization_flow(self) -> dict:
        """Initiates the Device Code Flow by calling the
        /device/authorize endpoint to retrieve the device_code and user_code.
        Provides Avalara device authorization code and related data

        Returns:
            dict: Dictionary with Avalara device authorization code
            and related data
        """
        return self.__get_device_authorization_user_code(
            self.oauth2_config.avalara_oidc_data.device_authorization_endpoint,
            self.client_id,
            self.client_secret,
        )

    def __get_access_token_data(
        self, url, client_id, client_secret, grant_type, device_code=None
    ) -> dict:
        data = {
            "grant_type": AVALARA_SDK_CONSTANTS.AVALARA_OIDC_VALUES_GRANT_TYPE_DEVICE_FLOW,
            "client_id": client_id,
            "client_secret": client_secret,
        }
        if device_code is not None:
            data["device_code"] = device_code
        encoded_data = urllib.parse.urlencode(data)
        http = urllib3.PoolManager()
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        response = http.request(
            'POST',
            url,
            headers=headers,
            body=encoded_data,
        )
        resp_dict = json.loads(response.data.decode('utf-8'))
        return resp_dict

    def __get_device_authorization_user_code(
        self, url, client_id, client_secret
    ) -> dict:
        data = {
            "grant_type": AVALARA_SDK_CONSTANTS.AVALARA_OIDC_VALUES_GRANT_TYPE_DEVICE_FLOW,
            "client_id": client_id,
            "client_secret": client_secret,
        }
        encoded_data = urllib.parse.urlencode(data)
        http = urllib3.PoolManager()
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        response = http.request(
            'POST',
            url,
            headers=headers,
            body=encoded_data,
        )
        resp_dict = json.loads(response.data.decode('utf-8'))
        return resp_dict



"""
AvaTax Software Development Kit for Python.

   Copyright 2022 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

    Avalara E-Invoicing API
    An API that supports sending data for an E-Invoicing compliance use-case. 

@author     Sachin Baijal <sachin.baijal@avalara.com>
@author     Jonathan Wenger <jonathan.wenger@avalara.com>
@copyright  2022 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    25.7.2
@link       https://github.com/avadev/AvaTax-REST-V3-Python-SDK
"""

import re  # noqa: F401
import sys  # noqa: F401
import decimal

from Avalara.SDK.api_client import ApiClient, Endpoint as _Endpoint
from Avalara.SDK.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from pydantic import Field, StrictBytes, StrictStr, field_validator
from typing import Optional, Union
from typing_extensions import Annotated
from Avalara.SDK.models.EInvoicing.V1.submit_interop_document202_response import SubmitInteropDocument202Response
from Avalara.SDK.exceptions import ApiTypeError, ApiValueError, ApiException
from Avalara.SDK.oauth_helper.AvalaraSdkOauthUtils import avalara_retry_oauth

class InteropApi(object):

    def __init__(self, api_client):
        self.__set_configuration(api_client)
    
    def __verify_api_client(self,api_client):
        if api_client is None:
            raise ApiValueError("APIClient not defined")
    
    def __set_configuration(self, api_client):
        self.__verify_api_client(api_client)
        api_client.set_sdk_version("25.7.2")
        self.api_client = api_client
		
        self.submit_interop_document_endpoint = _Endpoint(
            settings={
                'response_type': (SubmitInteropDocument202Response,),
                'auth': [
                    'Bearer'
                ],
                'endpoint_path': '/einvoicing/interop/documents',
                'operation_id': 'submit_interop_document',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'document_type',
                    'interchange_type',
                    'avalara_version',
                    'x_avalara_client',
                    'x_correlation_id',
                    'file_name',
                ],
                'required': [
                    'document_type',
                    'interchange_type',
                    'avalara_version',
                ],
                'nullable': [
                ],
                'enum': [
                    'document_type',
                    'interchange_type',
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                    ('document_type',): {

                        "&#39;invoice-2.1&#39;": 'ubl-invoice-2.1',
                        "&#39;creditnote-2.1&#39;": 'ubl-creditnote-2.1',
                        "&#39;applicationresponse-2.1&#39;": 'ubl-applicationresponse-2.1'
                    },
                    ('interchange_type',): {

                        "&#39;B2B-TIEKE&#39;": 'FI-B2B-TIEKE',
                        "&#39;B2G-TIEKE&#39;": 'FI-B2G-TIEKE'
                    },
                },
                'openapi_types': {
                    'document_type':
                        (str,),
                    'interchange_type':
                        (str,),
                    'avalara_version':
                        (str,),
                    'x_avalara_client':
                        (str,),
                    'x_correlation_id':
                        (str,),
                    'file_name':
                        (bytearray,),
                },
                'attribute_map': {
                    'document_type': 'documentType',
                    'interchange_type': 'interchangeType',
                    'avalara_version': 'avalara-version',
                    'x_avalara_client': 'X-Avalara-Client',
                    'x_correlation_id': 'X-Correlation-ID',
                    'file_name': 'fileName',
                },
                'location_map': {
                    'document_type': 'query',
                    'interchange_type': 'query',
                    'avalara_version': 'header',
                    'x_avalara_client': 'header',
                    'x_correlation_id': 'header',
                    'file_name': 'form',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'avalara-version': '1.3',
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'multipart/form-data'
                ]
            },
            api_client=api_client,
            required_scopes='',
            microservice='EInvoicing'
        )

    @avalara_retry_oauth(max_retry_attempts=2)
    def submit_interop_document(
        self,
        document_type,
        interchange_type,
        avalara_version,
        **kwargs
    ):
        """Submit a document  # noqa: E501

        This API used by the interoperability partners to submit a document to  their trading partners in Avalara on behalf of their customers.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.submit_interop_document(document_type, interchange_type, avalara_version, async_req=True)
        >>> result = thread.get()

        Args:
            document_type (str): Type of the document being uploaded. Partners will be configured in Avalara system to send only certain types of documents.
            interchange_type (str): Type of interchange (codes in Avalara system that uniquely identifies a type of interchange). Partners will be configured in Avalara system to send documents belonging to certain types of interchanges.
            avalara_version (str): The HTTP Header meant to specify the version of the API intended to be used

        Keyword Args:
            x_avalara_client (str): You can freely use any text you wish for this value. This feature can help you diagnose and solve problems with your software. The header can be treated like a \"Fingerprint\". [optional]
            x_correlation_id (str): The caller can use this as an identifier to use as a correlation id to trace the call.. [optional]
            file_name (bytearray): The file to be uploaded (e.g., UBL XML, CII XML).. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            SubmitInteropDocument202Response
                If the method is called asynchronously, returns the request
                thread.
        """
        self.__verify_api_client(self.api_client)
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['document_type'] = document_type
        kwargs['interchange_type'] = interchange_type
        kwargs['avalara_version'] = avalara_version
        return self.submit_interop_document_endpoint.call_with_http_info(**kwargs)




from typing import List


class AvalaraOidcModel:
    """Avalara OpenID Connect (OIDC) model"""

    issuer: str
    jwks_uri: str
    authorization_endpoint: str
    token_endpoint: str
    userinfo_endpoint: str
    end_session_endpoint: str
    check_session_iframe: str
    revocation_endpoint: str
    introspection_endpoint: str
    device_authorization_endpoint: str
    frontchannel_logout_supported: bool
    frontchannel_logout_session_supported: bool
    backchannel_logout_supported: bool
    backchannel_logout_session_supported: bool
    scopes_supported: List[str]
    claims_supported: List[str]
    grant_types_supported: List[str]
    response_types_supported: List[str]
    response_modes_supported: List[str]
    token_endpoint_auth_methods_supported: List[str]
    id_token_signing_alg_values_supported: List[str]
    subject_types_supported: List[str]
    code_challenge_methods_supported: List[str]
    request_parameter_supported: bool

    def __init__(self, json_obj=None) -> None:
        if json_obj:
            self.issuer = json_obj.issuer
            self.jwks_uri = json_obj.jwks_uri
            self.authorization_endpoint = json_obj.authorization_endpoint
            self.token_endpoint = json_obj.token_endpoint
            self.userinfo_endpoint = json_obj.userinfo_endpoint
            self.end_session_endpoint = json_obj.end_session_endpoint
            self.check_session_iframe = json_obj.check_session_iframe
            self.revocation_endpoint = json_obj.revocation_endpoint
            self.introspection_endpoint = json_obj.introspection_endpoint
            self.device_authorization_endpoint = (
                json_obj.device_authorization_endpoint
            )
            self.frontchannel_logout_supported = (
                json_obj.frontchannel_logout_supported
            )
            self.frontchannel_logout_session_supported = (
                json_obj.frontchannel_logout_session_supported
            )

            self.backchannel_logout_supported = (
                json_obj.backchannel_logout_supported
            )
            self.backchannel_logout_session_supported = (
                json_obj.backchannel_logout_session_supported
            )
            self.scopes_supported = json_obj.scopes_supported
            self.claims_supported = json_obj.claims_supported
            self.grant_types_supported = json_obj.grant_types_supported
            self.response_types_supported = json_obj.response_types_supported
            self.response_modes_supported = json_obj.response_modes_supported
            self.token_endpoint_auth_methods_supported = (
                json_obj.token_endpoint_auth_methods_supported
            )
            self.id_token_signing_alg_values_supported = (
                json_obj.id_token_signing_alg_values_supported
            )
            self.subject_types_supported = json_obj.subject_types_supported
            self.code_challenge_methods_supported = (
                json_obj.code_challenge_methods_supported
            )

            self.request_parameter_supported = (
                json_obj.request_parameter_supported
            )



"""
AvaTax Software Development Kit for Python.

   Copyright 2022 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

    Avalara E-Invoicing API
    An API that supports sending data for an E-Invoicing compliance use-case. 

@author     Sachin Baijal <sachin.baijal@avalara.com>
@author     Jonathan Wenger <jonathan.wenger@avalara.com>
@copyright  2022 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    25.7.2
@link       https://github.com/avadev/AvaTax-REST-V3-Python-SDK
"""

import re  # noqa: F401
import sys  # noqa: F401
import decimal

from Avalara.SDK.api_client import ApiClient, Endpoint as _Endpoint
from Avalara.SDK.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from pydantic import Field, StrictFloat, StrictInt, StrictStr
from typing import Optional, Union
from typing_extensions import Annotated
from Avalara.SDK.models.EInvoicing.V1.data_input_fields_response import DataInputFieldsResponse
from Avalara.SDK.exceptions import ApiTypeError, ApiValueError, ApiException
from Avalara.SDK.oauth_helper.AvalaraSdkOauthUtils import avalara_retry_oauth

class DataInputFieldsApi(object):

    def __init__(self, api_client):
        self.__set_configuration(api_client)
    
    def __verify_api_client(self,api_client):
        if api_client is None:
            raise ApiValueError("APIClient not defined")
    
    def __set_configuration(self, api_client):
        self.__verify_api_client(api_client)
        api_client.set_sdk_version("25.7.2")
        self.api_client = api_client
		
        self.get_data_input_fields_endpoint = _Endpoint(
            settings={
                'response_type': (DataInputFieldsResponse,),
                'auth': [
                    'Bearer'
                ],
                'endpoint_path': '/einvoicing/data-input-fields',
                'operation_id': 'get_data_input_fields',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'avalara_version',
                    'x_avalara_client',
                    'filter',
                    'top',
                    'skip',
                    'count',
                    'count_only',
                ],
                'required': [
                    'avalara_version',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'avalara_version':
                        (str,),
                    'x_avalara_client':
                        (str,),
                    'filter':
                        (str,),
                    'top':
                        (float,),
                    'skip':
                        (str,),
                    'count':
                        (bool,),
                    'count_only':
                        (bool,),
                },
                'attribute_map': {
                    'avalara_version': 'avalara-version',
                    'x_avalara_client': 'X-Avalara-Client',
                    'filter': '$filter',
                    'top': '$top',
                    'skip': '$skip',
                    'count': '$count',
                    'count_only': '$countOnly',
                },
                'location_map': {
                    'avalara_version': 'header',
                    'x_avalara_client': 'header',
                    'filter': 'query',
                    'top': 'query',
                    'skip': 'query',
                    'count': 'query',
                    'count_only': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'avalara-version': '1.3',
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            required_scopes='',
            microservice='EInvoicing'
        )

    @avalara_retry_oauth(max_retry_attempts=2)
    def get_data_input_fields(
        self,
        avalara_version,
        **kwargs
    ):
        """Returns the optionality of document fields for different country mandates  # noqa: E501

        This endpoint provides a list of required, conditional, and optional fields for each country mandate. You can use the <code>mandates</code> endpoint to retrieve all available country mandates. You can use the $filter query parameter to retrieve fields for a particular mandate  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_data_input_fields(avalara_version, async_req=True)
        >>> result = thread.get()

        Args:
            avalara_version (str): The HTTP Header meant to specify the version of the API intended to be used

        Keyword Args:
            x_avalara_client (str): You can freely use any text you wish for this value. This feature can help you diagnose and solve problems with your software. The header can be treated like a fingerprint.. [optional]
            filter (str): Filter by field name and value. This filter only supports <code>eq</code> and <code>contains</code>. Refer to [https://developer.avalara.com/avatax/filtering-in-rest/](https://developer.avalara.com/avatax/filtering-in-rest/) for more information on filtering.. [optional]
            top (float): The number of items to include in the result.. [optional]
            skip (str): If nonzero, skip this number of results before returning data. Used with <code>$top</code> to provide pagination for large datasets.. [optional]
            count (bool): When set to true, the count of the collection is also returned in the response body. [optional]
            count_only (bool): When set to true, only the count of the collection is returned. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            DataInputFieldsResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        self.__verify_api_client(self.api_client)
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['avalara_version'] = avalara_version
        return self.get_data_input_fields_endpoint.call_with_http_info(**kwargs)




# coding: utf-8

"""
AvaTax Software Development Kit for Python.

   Copyright 2022 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

    Avalara E-Invoicing API
    An API that supports sending data for an E-Invoicing compliance use-case. 

@author     Sachin Baijal <sachin.baijal@avalara.com>
@author     Jonathan Wenger <jonathan.wenger@avalara.com>
@copyright  2022 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    25.7.2
@link       https://github.com/avadev/AvaTax-REST-V3-Python-SDK
"""

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from Avalara.SDK.models.EInvoicing.V1.error_response import ErrorResponse
from typing import Optional, Set
from typing_extensions import Self

class BatchSearch(BaseModel):
    """
    Provides details about a batch search operation, including status, timestamps, and potential errors.
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, description="ID of the batch search")
    name: Optional[StrictStr] = Field(default=None, description="Name of the batch report")
    created_by: Optional[StrictStr] = Field(default=None, description="Email of the user who created the batch search", alias="createdBy")
    created: Optional[datetime] = Field(default=None, description="Timestamp when the batch search was created")
    last_modified: Optional[datetime] = Field(default=None, description="Timestamp when the batch search was created", alias="lastModified")
    status: Optional[StrictStr] = Field(default=None, description="Status of the batch search")
    error: Optional[ErrorResponse] = None
    __properties: ClassVar[List[str]] = ["id", "name", "createdBy", "created", "lastModified", "status", "error"]

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Accepted', 'Running', 'Completed', 'Failed']):
            raise ValueError("must be one of enum values ('Accepted', 'Running', 'Completed', 'Failed')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BatchSearch from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of error
        if self.error:
            _dict['error'] = self.error.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BatchSearch from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "name": obj.get("name"),
            "createdBy": obj.get("createdBy"),
            "created": obj.get("created"),
            "lastModified": obj.get("lastModified"),
            "status": obj.get("status"),
            "error": ErrorResponse.from_dict(obj["error"]) if obj.get("error") is not None else None
        })
        return _obj


